﻿;;;-*- Mode: Lisp; Package: "CSCWD-ONTO" -*-
;;;===============================================================================
;;;14/11/03
;;;             AGENT CSCWD-ONTO: ONTOLOGY (file CSCWD-ONTO-ONTOLOGY.lisp)
;;;             Copyright Barthès@UTC, 2014
;;;
;;;=============================================================================== 
#|
PUBLISHER is a multilingual service agent. IN A MULTILINGUAL ENVIRONMENT
AVOID USING HAS-xxx MACROS. INSTEAD USE (send <obj-id> '=get "property
name"). The xxx of the HAS-xxx macro is the first English synonym when
the language is :ALL. Otherwise it is the first synonym of *language* 

Its ontology is used to process index news items into a database of news
items
   
2014
 0920 Creation
 0924 adding annotations
 0930 new if-needed methods for cscwd concept (uid and role)
 1103 changing data to creation date for annotations
 1112 introducing role types
|#

(in-package :CSCWD-ONTO)

(defontology 
  (:title "CSCWD-ONTO ontology")
  (:version "1.0")
  (:language :en)
  )

;;;===============================================================================
;;;                              ANNOTATION
;;;===============================================================================

(defconcept "Annotation"
    (:doc "An object containing some remarks about another object.")
  (:att "text")
  (:att "title")
  (:rel "author" (:to "Person"))
  (:att "creation date")
  ;; language allows to filter annotation objects for posting them according to
  ;; their language (filled automatically)
  (:att "language")
  )

;;;===============================================================================
;;;                                CONCEPT
;;;===============================================================================

(defconcept "CSCWD Concept"
  (:doc "A concept in the CSCWD ontology")
  ;;(:att "uid")
  (:att "label" (:entry))
  (:att (:en "definition" :fr "définition"))
  (:att (:en "reference" :fr "référence"))
  (:att (:en "creation date" :fr "date de création"))
  (:att (:en "modification date" :fr "date de modification"))
  (:rel (:en "author" :fr "auteur") (:to "person"))
  (:att (:en "source" :fr "source")) ; a set of pointers as an MLN value
  (:rel (:en "subsuming concept" :fr "super-type") (:to "CSCWD Concept"))
  (:rel (:en "annotation" :fr "annotation") (:to "Annotation"))
  ;;(:rel "role" (:to "Role"))
  )
  
;;;===============================================================================
;;;                                PERSON
;;;===============================================================================

(defconcept "Person"
    (:doc 
     "A PERSON in our case is limited to a name, first-name, initials, email ~
      and PA.")
  (:att (:en "name" :fr "nom") (:entry))
  (:att (:en "first name" :fr "prénom") (:entry))
  (:att (:en "initials" :fr "initiales") (:entry))
  (:att (:en "email" :fr "email"))
  (:att (:en "PA; personal assistant" :fr "AP;assistant personnel"))
  )
  
;;;===============================================================================
;;;                                Role
;;;===============================================================================

(defconcept "Role"
    (:doc "A ROLE is a link between two CSCWD concepts")
  (:rel (:en "role type" :fr "type de role")(:to "Role Type")(:min 1))
  (:att (:en "creation date" :fr "date de création"))
  (:att (:en "modification date" :fr "date de modification"))
  (:rel (:en "author" :fr "auteur") (:to "person"))
  (:rel (:en "from" :fr "origine") (:to "CSCWD Concept"))
  (:rel (:en "target" :fr "cible") (:to "CSCWD Concept"))
  )

;;;===============================================================================
;;;                               Role Type
;;;===============================================================================

(defconcept "Role Type"
    (:doc "A ROLE TYPE is a specific kind of ROLE.")
  (:att "label" (:entry))
  (:att (:en "definition" :fr "définition"))
  (:att (:en "creation date" :fr "date de création"))
  (:att (:en "modification date" :fr "date de modification"))
  (:rel (:en "author" :fr "auteur") (:to "person"))
  )
  
;;;===============================================================================
;;;
;;;                                METHODS
;;;
;;;===============================================================================

;;;------------------------------------------------------- (PERSON) =parse-summary
;;; initials could be characterized by the fact that their length is twice the
;;; number of dots in the string after removing hyphens "-"  and equal signs "="
;;; But since this depends on cultures we require that first names do not contain
;;; dots

(defownmethod =parse-summary $E-PERSON (val)
  "takes a value like \"Barthès: Jean-Paul\" or \"Moulin: C.\" and returns the ~
   list ((\"name\" \"Barthès\") (\"first-name\" \"Jean-Paul\")) or
        ((\"name\" \"Moulin\")(\"initials\" \"C.\"))
We assume here that the first name does not contain dots like in \"Jean-Paul A.\" ~
as can be found if the middle name is attached to the first name."
  (let (name first-name initials pos rest)
    ;; val must be a string
    (when (stringp val)
      (setq val (string-trim '(#\space) val))
      ;; extract name first
      ;; get the position of column
      (setq pos (position #\: val))
      (cond
       (pos
        (setq name (string-trim '(#\space) (subseq val 0 pos)))
        ;; extract the rest
        (setq rest (string-trim '(#\space) (subseq val  (1+ pos))))
        ;; now try to figure out if a first name or initials
        ;; if no dot present, first name
        (setq pos (position #\. rest))
        (if pos
            ;; might be initials
            (progn
              (setq initials rest)
              )
          ;; must be first name
          (setq first-name (if (equal+ rest "") nil rest)))
        
        ;; return description
        `(("name" ,name)
          ,@(if first-name `(("first-name" ,first-name)))
          ,@(if initials `(("initials" ,initials)))))
       ;; if no ":" and not empty string assume it is the name
       ((not (string-equal val ""))
        `(("name" ,val)))
       ))))
                  
#|
(send '$e-person '=parse-summary "Barthès: Jean-Paul")
(("name" "Barthès") ("first-name" "Jean-Paul"))
(send '$e-person '=parse-summary "Moulin: C.")
(("name" "Moulin") ("initials" "C."))
(send '$e-person '=parse-summary "Barthès")
(("name" "Barthès"))
(send '$e-person '=parse-summary "DBB :")
(("name" "DBB"))
|#
;;;========================================================= (annotation) =summary

(definstmethod =summary annotation ()
  (list (format nil "~A ~A:~{~A ~} ~{~A~^, ~}"
          (car (send (car (HAS-author)) '=summary))
          (HAS-creation-DATE)
          (HAS-title)
          (HAS-text)
          )))

#|
(send '$e-annotation.1 '=summary)
("Barthès : Jean-Paul (30/10/2014): Sample annotation 1 for tests")
|#
;;;====================================================== (cscwd-concept) =summary

(definstmethod =summary cscwd-concept ()
  (let ((val (car (send *self* '=get "label"))))
    (if val
        (list (mln::extract-to-string val :always t))
      (list "?"))))

#|
(send '$E-CSCWD-CONCEPT.3 '=summary)
("Travail coopératif assisté par ordinateur pour le desing, TCAOD")
(let ((*language* :en)) (send '$E-CSCWD-CONCEPT.3 '=summary))
("Computer-Supported Cooperative Work in Design, CSCWD")
|#
;;;=============================================================== (role) =summary

(definstmethod =summary role ()
  (let* ((val (car (send *self* '=get "label")))
         (from (car (send *self* '=get "from")))
         (from-identity (car (send from '=summary)))
         (target (send *self* '=get "target"))
         (target-identity (broadcast target '=summary))
         )
    ;(format t "~%; =summary for role / val: ~S" VAL)
    ;(format t "~%; =summary for role / from-identity: ~S" from-identity)
    ;(format t "~%; =summary for role / target-identity: ~S" target-identity)
    (if val
        (list (format nil "~A/~A/~{~{~A~}~^, ~}"
                from-identity
                (mln::extract-to-string val :always t)
                target-identity
                ))
      (list "?"))))

#|
(send '$E-ROLE.1 '=summary)
("Theory/includes/Concept, Knowledge")
(let ((*language* :FR)) (send '$E-ROLE.1 '=summary))
("Théorie/comprend, est composé de/Concept, Connaissances")
|#
;;;========================================================== (role-type) =summary

(definstmethod =summary role-type ()
  (let ((val (car (send *self* '=get "label"))))
    (format t "~%; =summary for role-type / val: ~S" VAL)
    (if val
        (list (mln::extract-to-string val :always t))
      (list "?"))))

#|
(send '$E-ROLE-TYPE.1 '=summary)
("includes")
(let ((*language* :FR)) (send '$E-ROLE-TYPE.1 '=summary))
("comprend, est composé de")
|#
;;;-------------------------------------------------- (cscwd-concept) =if-needed
;;(defownmethod =if-needed 
;;  ("uid" "MOSS PROPERTY-NAME" "MOSS ATTRIBUTE" :class-ref "CSCWD Concept") 
;;  (obj-id) 
;;  (let ((uid (moss::%get-entry-point-if-unique obj-id)))
;;    (list uid)))

;;;-------------------------------------------------- (cscwd-concept) =if-needed
;;; The following method returns a list of roles where the concept is the source

;;(defownmethod =if-needed
;;    ("role" "MOSS PROPERTY-NAME" "MOSS RELATION" :class-ref "CSCWD Concept")
;;  (obj-id)
;;  (let (uid)
;;    (setq uid (car (send obj-id '=get "uid")))
;;    (access `("Role" ("from" ("CSCWD Concept" ("uid" :is ,uid)))))))
    
;;;============================================================= (person) =summary

(definstmethod =summary person ()
  (list (format nil "~{~A ~}: ~{~A~^, ~}" 
          (send *self* '=get "name")
          (send *self* '=get "first-name")
          )))
#|
(send '$e-person.1 '=summary)
("Barthès : Jean-Paul")
|#


;;;===============================================================================
;;;===============================================================================
;;;
;;;                                INDIVIDUALS
;;;
;;;===============================================================================
;;;=============================================================================== 

;;;====================================================================== PERSONNE

(defmacro defperson (name first-name initials &optional email)
  `(defindividual "person"
       ("name" ,@(if (listp name) name (list name)))
     ("first name" ,@(if (listp first-name) first-name (list first-name)))
     ("initials" ,initials)
     ("email" ,email)
     ))
  
(defperson "Barthès" "Jean-Paul" "JPB" "barthes@utc.fr")
(defperson "Gonzales" "Omar" "OPG" "omar@cinvestav.mx")
(defperson "Ramos" "Milton" "MR" "milton@tecpar.br")

;;;====================================================================== CONCEPTS
#|
(defconcept (:en "Computer-Supported Cooperative Work in Design; CSCWD"
              :fr "Travail coopératif assisté par ordinateur pour le desing; TCAOD")
  (:doc :en "This designate a specialization of CSCWD applied to Design."
      :fr "Ceci designe une spécialisation du TCAO à la conception.")
	  )
|#

(defindividual "CSCWD Concept"
   ("label" 
      ((:en "Root") (:fr "Racine")))
   ("definition"
      ((:en "The ROOT concept is an artificial construct to root the subsumption tree.")
	   (:fr "Le concept de RACINE est artificiel et sert de racine à l'arbre de subsumption")))
   ("author" "barthès")
   ("creation date" "21/9/2014")
   )

(defindividual "CSCWD Concept"
    ("label" 
     ((:en "Computer-Supported Cooperative Work" "CSCW")
      (:fr "Travail coopératif assisté par ordinateur" "TCAO")))
  ("definition"   
   ((:en "CSCW is a generic term, which combines the understanding of the way people work in groups with the enabling technologies of computer networking, and associated hardware, software, services and techniques.")
    (:fr "Le TRAVAIL CCOPERATIF ASSISTE PAR ORDINATEUR (TCAO) est une méthode de travail permettant à des utilisateurs de travailler en groupe et en réseau, associant des matériels, logiciels, techniques et méthodes spécifiques.")))
  ("source" ((:en "http://en.wikipedia.org/wiki/Computer-supported_cooperative_work")
             (:fr "http://fr.wikipedia.org/wiki/Travail_coop%C3%A9ratif_assist%C3%A9_par_ordinateur")))
  ("author" "barthès")
  ("creation date" "21/09/2014")
  ("subsuming concept" "ROOT")
  ("annotation"
   (:new "annotation" ("author" "barthès")("date" "30/10/2014")
         ("text" "Sample annotation 1 for tests")("language" "EN"))
   (:new "annotation" ("author" "barthès")("date" "30/10/2014")
         ("text" "Annotation 2 à fin de test"))
   )
  )

	  
(defindividual "CSCWD Concept"
   ("label" ((:en "Computer-Supported Cooperative Work in Design" "CSCWD")
              (:fr "Travail coopératif assisté par ordinateur pour le desing"  "TCAOD")))
   ("definition" ((:en "This designate a specialization of CSCWD applied to Design.")
      (:fr "Ceci designe une spécialisation du TCAO à la conception.")
	  ))
   ("subsuming concept" "CSCW")
   ("author" "barthès")
   ("creation date" "21/09/2014")
  )

(defmacro defc (title e-def &optional titre f-def isa source f-source)
  `(defindividual "CSCWD Concept"
       ("label" ((:en ,@(omas::split-val title))
                 ,@(if titre `((:fr ,@(omas::split-val titre))))))
     ("definition" ((:en ,e-def) ,@(if f-def `((:fr ,f-def)))))
     ,@(if (or source f-source)
           `(("source" (,@(if source `((:en ,source)))
                             ,@(if f-source `((:fr ,f-source)))))))
     ,@(if isa `(("subsuming concept" ,isa)))
     ("author" "barthès")
     ("creation date" "27/09/2014")
     ))
 
(defc "Administration" "Central administration, the highest administrative department of an organization." 
  "Administration" "On appelle administration (ou administration publique) l’ensemble des services chargés d’assurer le fonctionnement d’un État, d’une collectivité territoriale ou d’un service public, et qui sont financés principalement par des prélèvements fiscaux autorisés par le Vote d'un Budget."
  "Root")
(defc "Aerospace" "Aerospace describes the human effort in science, engineering and business to fly in the atmosphere of Earth (aeronautics) and surrounding space (astronautics)." 
  "Aéronautique et espace" "L'aérospatiale (nom commun féminin singulier) est une discipline scientifique qui rassemble les techniques de l'aéronautique (déplacement dans l'atmosphère, utilisant des avions ou des hélicoptère par exemple) et spatiales (déplacements spatiaux, c'est-à-dire trajets hors atmosphère et interplanétaires, en utilisant des navettes spatiales ainsi que des fusées)."
  "Industry")
(defc "Agent" "In computer science, a software agent is a computer program that acts for a user or other program in a relationship of agency: an agreement to act on one's behalf." 
  "Agent" "Un agent est une entité caractérisée par le fait qu'elle est, au moins partiellement, autonome. Ce peut être un processus, un robot, un être humain, etc"
  "Software")
(defc "Application" "Application software is a set of one or more programs designed to carry out operations for a specific application. Application software cannot run on itself but is dependent on system software to execute." 
  "Application" "Une application, dans le domaine informatique, est à la fois une activité pour laquelle des moyens informatiques sont utilisés et le moyen informatique utilisé pour l'activité en question"
  "Software")
(defc "Architecture" "Software architecture is the high level structure of a software system, the discipline of creating such a high level structure, and the documentation of this structure."
   "Architecture" "L’architecture logicielle décrit d’une manière symbolique et schématique les différents éléments d’un ou de plusieurs systèmes informatiques, leurs interrelations et leurs interactions. Contrairement aux spécifications produites par l’analyse fonctionnelle, le modèle d'architecture, produit lors de la phase de conception, ne décrit pas ce que doit réaliser un système informatique mais plutôt comment il doit être conçu de manière à répondre aux spécifications"
   "Concept")
(defc "Automotive Industry" "The automotive industry is a wide range of companies and organizations involved in the design, development, manufacture, marketing, and selling of motor vehicles." 
  "Industrie automobile" "La construction automobile est la branche d'activité économique qui rassemble les activités de conception, de fabrication et de commercialisation des véhicules de type tourisme et poids-lourds"
  "Industry")
(defc "Big Data" "Big data is an all-encompassing term for any collection of data sets so large and complex that it becomes difficult to process using traditional data processing applications." 
  "Big Data, datamasse" "Les big data, littéralement les « grosses données », ou mégadonnées (recommandé), parfois appelées données massives, désignent des ensembles de données qui deviennent tellement volumineux qu'ils en deviennent difficiles à travailler avec des outils classiques de gestion de base de données ou de gestion de l'information."
  "Data structure")
(defc "Business" "A business is a legally recognized organizational entity designed to provide goods or services" "Affaires, Commerce" "..."
  "Root")
(defc "Business Process" "A business process or business method is a collection of related, structured activities or tasks that produce a specific service or product (serve a particular goal) for a particular customer or customers."
  "Procédure" "Une procédure est la manière spécifiée d'effectuer une activité ou un processus"
  "Method")
(defc "Cloud, Cloud Computing" "Cloud computing is internet-based computing in which large groups of remote servers are networked to allow sharing of data-processing tasks, centralized data storage, and online access to computer services or resources." 
  "Le nuage, Cloud, Cloud computing" "Le cloud computing, abrégé en cloud (le Nuage en français) ou l’informatique en nuage désigne un ensemble de processus qui consiste à utiliser la puissance de calcul et/ou de stockage de serveurs informatiques distants à travers un réseau, généralement Internet" 
  "Computer tool")
(defc "Collaboration" "Collaboration is working with others to do a task and to achieve shared goals. It is a recursive process where two or more or organizations work together to realize shared goals, (this is more than the intersection of common goals seen in co-operative ventures, but a deep, collective determination to reach an identical objective)." 
  "Collaboration" "La collaboration est l'acte de travailler ensemble pour atteindre un objectif. Dans son sens commun, la collaboration est un processus par lequel deux ou plusieurs personnes ou organisations s’associent pour réaliser un travail suivant des objectifs communs."
  "Root")
(defc "Collaboration Methods" "Methods that can help to organize collaborative work." 
  "Méthodes de collaboration" "Méthodes qui permettent d'organiser le travail collaboratif."
  "Method")
(defc "Collaboration Theories" "..." "Théories de la collaboration" "..."
  "Collaboration"
  "Concept")
(defc "Collaboration Methodologies" "..." "Méthodologie pour la collaboration" "..."
  "Method")
(defc "Collaborative Computing" "...")
(defc "Collaborative Platforms" "A collaboration platform is a category of business software that adds broad social networking capabilities to work processes." 
  "Plate-Forme collaborative" "Une plate-forme de travail collaboratif est un espace de travail virtuel. C'est un outil, parfois sous la forme d'un site Internet, qui centralise tous les outils liés à la conduite d'un projet, la gestion des connaissances ou au fonctionnement d'une organisation1 et les met à disposition des acteurs."
  "Computer Tool")
(defc "Collaborative Processing of Big Data" "...")
(defc "Collaborative Virtual Environments" "..." 
  "Environnements virtuels collaboratifs" "..."
  "Computer tool")
(defc "Collaborative Workflow" "Collaborative workflow is the convergence of social software with service management (workflow) software.")
(defc "Collaborative Workspace" "A collaborative workspace or shared workspace is an inter-connected environment in which all the participants in dispersed locations can access and interact with each other just as inside a single entity."
  "Espaces collaboratifs" "...")
(defc "Computer Tool" "..." 
  "Outil informatique" "..."
  "Software")
(defc "Concept" "A concept is an abstraction or generalization from experience or the result of a transformation of existing concepts. The concept reifies all of its actual or potential instances whether these are things in the real world or other ideas." 
  "Concept" "Un concept est une représentation générale et abstraite de la réalité d'un objet, d'une situation ou d'un phénomène."
  "Root")
(defc "Construction" "Building construction is the process of preparing for and forming buildings and building systems." 
  "Entreprise du bâtiment" "..."
  "Industry")
(defc "Data Structure" "In computer science, a data structure is a particular way of organizing data in a computer so that it can be used efficiently" 
   "Structure de données" "En informatique, une structure de données est une structure logique destinée à contenir des données, afin de leur donner une organisation permettant de simplifier leur traitement" 
   "Software")
(defc "Device" "A device is usually a constructed tool. Here it may be a component of a computer hardware or an electronic instrument."
  "Dispositif, Appareil" "Un dispositif est un appareil qui peut être composé de matériel et de logiciel permettant de réaliser une fonction." 
  "Hardware")
(defc "Engineering" "Engineering is the application of scientific, economic, social, and practical knowledge in order to invent, design, build, maintain, and improve structures, machines, devices, systems, materials and processes"
  "Ingénierie" "L'ingénierie désigne l'ensemble des fonctions qui mènent de la conception et des études, de l'achat et du contrôle de fabrication des équipements, à la construction et à la mise en service d'une installation technique ou industrielle"
  "Root")
(defc "Entertainment Industry" "Sometimes referred to as show business, the industry relies on business models to produce, market, broadcast or otherwise distribute many of its traditional forms, including performances of all types." 
  "Industrie du spectacle" "L'industrie du spectacle ou industrie du divertissement recouvre l’ensemble de l'industrie du divertissement."
  "Industry")
(defc "Facility Management" "Facility management (or facilities management or FM) is an interdisciplinary field devoted to the coordination of space, infrastructure, people and organization, often associated with the administration of office blocks, arenas, schools, convention centers, shopping complexes, hospitals, hotels, etc." 
  "Multi-services" "Désigne une externalisation de l'ensemble des services nécessaires au fonctionnement normal d'une entreprise."
  "Management method")
(defc "Grid, Grid Computing" "Grid computing is the collection of computer resources from multiple locations to reach a common goal. The grid can be thought of as a distributed system with non-interactive workloads that involve a large number of files" 
  "Grille informatique" "Une grille informatique (en anglais, grid) est une infrastructure virtuelle constituée d'un ensemble de ressources informatiques potentiellement partagées, distribuées, hétérogènes, délocalisées et autonomes." 
  "Software Tool")
(defc "Hardware" "..." "Matériel" "..."
  "Root")
(defc "Healthcare" "Health care (or healthcare) is the diagnosis, treatment, and prevention of disease, illness, injury, and other physical and mental impairments in human beings."
  "Santé" "Les soins de santé relèvent des sciences appliquées et des sciences biomédicales. Ils se centrent sur la santé des humains ou des animaux."
  "Root")
(defc "Homecare" "Home care, (also referred to as domiciliary care, social care, or in-home care), is supportive care provided in the home. Care may be provided by licensed healthcare professionals who provide medical care needs or by professional caregivers who provide daily care to help to ensure the activities of daily living (ADL's) are met."
  "Hospitalisation à domicile" "L'hospitalisation à domicile, nommée également par l'acronyme HAD, est une alternative à l'hospitalisation classique intra-muros. Assurée par des organismes ayant le statut d'établissement de santé, ils coordonnent au domicile du patient la réalisation de soins médicaux et paramédicaux lourds, ou nécessitant l'utilisation de matériels ou thérapeutiques ordinairement réservés à l'usage hospitalier1."
  "Healthcare")
(defc "Infrastructure" "Infrastructure is the basic physical and organizational structure needed for the operation of a society or enterprise ,[1] or the services and facilities necessary for an economy to function. Communication infrastructure: The Internet, including the internet backbone, core routers and server farms, local internet service providers as well as the protocols and other basic software required for the system to function (does not include specific websites, although may include some widely used web-based services, such as social network services and web search engines)."
  "Infrastructure" "L'infrastructure est un ensemble d'éléments structuraux interconnectés qui fournissent le cadre pour supporter la totalité de la structure. Le terme est souvent utilisé d'une façon très abstraite. Par exemple, les outils d'ingénierie informatique sont quelquefois décrits comme une partie de l'infrastructure d'un environnement de développement,"
  "Root")
(defc "Intelligent Building" "..." "Immeubles intelligents" "..."
  "Industry")
(defc "Internet" "The Internet is a global system of interconnected computer networks that use the standard Internet protocol suite (TCP/IP) to link several billion devices worldwide. It is an international network of networks that consists of millions of private, public, academic, business, and government packet switched networks, linked by a broad array of electronic, wireless, and optical networking technologies." 
  "Internet" "nternet est le réseau informatique mondial accessible au public. C'est un réseau de réseaux, sans centre névralgique, composé de millions de réseaux aussi bien publics que privés, universitaires, commerciaux et gouvernementaux. L'information est transmise par Internet grâce à un ensemble standardisé de protocoles de transfert de données, qui permet l'élaboration d'applications et de services variés comme le courrier électronique, la messagerie instantanée, le pair-à-pair et le World Wide Web."
  "Software")
(defc "Internet of Things" "The Internet of Things (IoT) is the interconnection of uniquely identifiable embedded computing devices within the existing Internet infrastructure. Typically, IoT is expected to offer advanced connectivity of devices, systems, and services that goes beyond machine-to-machine communications (M2M) and covers a variety of protocols, domains, and applications."
  "Internet des objets" "L'Internet des objets représente l'extension d'Internet à des choses et à des lieux du monde physique. Alors qu'Internet ne se prolonge habituellement pas au-delà du monde électronique, l'internet des objets (IdO, ou IoT en anglais) représente les échanges d'informations et de données provenant de dispositifs présents dans le monde réel vers le réseau Internet."
  "Internet")
(defc "Interoperability" "Interoperability is the ability of making systems and organizations work together (inter-operate)." 
  "Interopérabilité" "L'interopérabilité est la capacité que possède un produit ou un système, dont les interfaces sont intégralement connues, à fonctionner avec d'autres produits ou systèmes existants ou futurs et ce sans restriction d'accès ou de mise en œuvre."
  "Concept")
(defc "Knowledge" "Facts, information, and skills acquired through experience or education; the theoretical or practical understanding of a subject:"
  "Connaissances" "La connaissance est une notion aux sens multiples à la fois utilisée dans le langage courant et objet d'étude poussée de la part des philosophes contemporains."
  "Root")
(defc "Knowledge Management" "Knowledge management (KM) is the process of capturing, developing, sharing, and effectively using organisational knowledge. It refers to a multi-disciplined approach to achieving organisational objectives by making the best use of knowledge."
 "Capitalisation des connaissances" "La gestion des connaissances (en anglais Knowledge Management) est l'ensemble des initiatives, des méthodes et des techniques permettant de percevoir, d'identifier, d'analyser, d'organiser, de mémoriser, et de partager des connaissances entre les membres des organisations, en particulier les savoirs créés par l'entreprise elle-même (par exemple marketing, recherche et développement) ou acquis de l'extérieur (par exemple intelligence économique) en vue d'atteindre l'objectif fixé."
  "Knowledge")
(defc "Knowledge Intensive Approach" "An approach relying heavily on knowledge." "Approche fondée sur la connaissance" "Un approche s'appuyant sur les connaissances."
  "Knowledge")
(defc "Logistics" "Logistics is the management of the flow of goods between the point of origin and the point of consumption in order to meet some requirements, of customers or corporations. The resources managed in logistics can include physical items, such as food, materials, animals, equipment and liquids, as well as abstract items, such as time, information, particles, and energy. The logistics of physical items usually involves the integration of information flow, material handling, production, packaging, inventory, transportation, warehousing, and often security." 
  "Logistique" "La logistique est l'activité qui a pour objet de gérer les flux physiques (ainsi que les données informatives ou financières s'y rapportant) d'une organisation, dans le but de mettre à disposition les ressources correspondant à des besoins (plus ou moins) déterminés."
  "Management method")
(defc "Mnagement method" "..." "Méthode de management" "..."
   "Method")
(defc "Manufacturing" "Manufacturing is the production of merchandise for use or sale using labor and machines, tools, chemical and biological processing, or formulation. The term may refer to a range of human activity, from handicraft to high tech, but is most commonly applied to industrial production, in which raw materials are transformed into finished goods on a large scale."
  "Industrie manufacturière" "L’industrie manufacturière est un sous-ensemble du secteur secondaire, et un sous-ensemble de l’industrie (sachant que toute l’industrie ne fait pas partie du secteur secondaire). L’industrie manufacturière regroupe les industries de transformation des biens, mais aussi la réparation et l'installation d'équipements industriels ainsi que des opérations en sous-traitance"
  "Industry")
(defc "Method" "..." 
  "Méthode" "Une méthode peut être : une démarche systématique, qui permet de décomposer le thème d'étude en tâches simples, et de faciliter la comparaison de l'étude avec d'autres études similaires ; une procédure à appliquer pas à pas, éventuellement avec une liste de contrôle (check list) dont on coche les étapes à chaque fois qu'elles sont finies ; le dernier avatar de cette méthode sont les procédures automatisées informatiques de type script (pas d'interaction de l'utilisateur) ou les assistants (l'utilisateur a des choix à faire, des actions à mener, et valide les étapes)."
  "Root")
(defc "Methodology" "Methodology is the systematic, theoretical analysis of the methods applied to a field of study. It comprises the theoretical analysis of the body of methods and principles associated with a branch of knowledge. Typically, it encompasses concepts such as paradigm, theoretical model, phases and quantitative or qualitative techniques"
  "Méthodologie" "La méthodologie est littéralement la science (logos) de la méthode, le discours (logos) sur la méthode, la cartographie des méthodes ou tout simplement la métaméthode ou méthode des méthodes. La méthodologie est une classe de méthodes, une sorte de boite à outils où chaque outil est une méthode de la même catégorie."
  "Method")
(defc "Multi-Agent Systems" "A multi-agent system (M.A.S.) is a computerized system composed of multiple interacting intelligent agents within an environment." 
  "Système multi-agents" "En informatique, un système multi-agent (SMA) est un système composé d'un ensemble d'agents, situés dans un certain environnement et interagissant selon certaines relations"
  "Software")
(defc "Computer network" "A computer network or data network is a telecommunications network that allows computers to exchange data." 
  "Réseau informatique" "Un réseau informatique est un ensemble d'équipements reliés entre eux pour échanger des informations"
  "Computer tool")
(defc "Ontology" "In computer science and information science, ontologies are used to formally represent knowledge within a domain. An ontology is defined as a formal, explicit specification of a shared conceptualization." 
  "Ontologie" "En informatique une ontologie est l'ensemble structuré des termes et concepts représentant le sens d'un champ d'informations, que ce soit par les métadonnées d'un espace de noms, ou les éléments d'un domaine de connaissances"
  "Knowledge")
(defc "Platform" "Computing platform, a framework on which applications may be run" 
  "Plate-forme, Plateforme" "Une plate-forme est en informatique un dispositif matériel et logiciel à partir duquel on peut développer un ensemble d'applications."
  "Computer Tool")
(defc "Power, Energy" "Energy economics is a broad scientific subject area which includes topics related to supply and use of energy in societies.[1] Due to diversity of issues and methods applied and shared with a number of academic disciplines, energy economics does not present itself as a self-contained academic discipline, but it is an applied subdiscipline of economics." 
  "Secteur de l'énergie" "L'économie de l'énergie décrit l'approvisionnement en énergie des différents agents économiques et l'activité de l'ensemble des entreprises qui produisent, commercialisent et distribuent de l’énergie. Elle regroupe la production et la consommation d’énergie et on y distingue d'une part l'exploitation des sources d'énergie et d'autre part la production et la distribution. Cela comprend donc la production d'électricité, la distribution d'électricité, la production de produits pétroliers et celle de gaz naturel."
  "Industry")
(defc "Privacy" ".La connaissance est une notion aux sens multiples à la fois utilisée dans le langage courant et objet d'étude poussée de la part des philosophes contemporains."
  "Vie privée" "On traduit en français l'expression anglaise de privacy par respect de la vie privée. Ce concept comporte plusieurs aspects. Le plus indiscutable est celui de ne pas être ennuyé avec le spam sur Internet, ni avec les publipostages dans la vie courante. Un autre aspect est que personne n'aime se dire que ses goûts et opinions peuvent être tracés, car certains organismes pourraient en faire un mauvais usage."
  "Concept")
(defc "Protocol" "In telecommunications, a communications protocol is a system of digital rules for data exchange within or between computers."
  "Protocole" "Dans les réseaux informatiques et les télécommunications, un protocole de communication est une spécification de plusieurs règles pour un type de communication particulier."
  "Method")
(defc "Security" "Security is the degree of resistance to, or protection from, harm. It applies to any vulnerable and valuable asset, such as a person, dwelling, community, nation, or organization."
  "Sûreté" "La sûreté est l'état de ce qui est sûr. De façon générale, il s'agit d'un état de protection contre le danger ou les menaces"
  "Concept")
(defc "Semantics" "Semantics is the study of meaning. It can be used to organize information." 
   "Sémantique" "La sémantique est une branche de la linguistique qui étudie les signifiés, soit, ce dont parle un énoncé. La sémantique tient le rôle d'une cartographie de l'information, elle permet de situer les informations les unes par rapport aux autres."
   "Concept")
(defc "Smart Homes" "Smart Home is the result of home automation. Home automation is the residential extension of building automation."
  "Domotique" "La domotique est l’ensemble des techniques de l'électronique, de physique du bâtiment, d'automatisme, de l'informatique et des télécommunications utilisées dans les bâtiments, plus ou moins « interopérables » et permettant de centraliser le contrôle des différents systèmes et sous-systèmes de la maison et de l'entreprise (chauffage, volets roulants, porte de garage, portail d'entrée, prises électriques, etc.)."
  "Industry")
(defc "Social Networks" "A social network is a social structure made up of a set of social actors (such as individuals or organizations) and a set of the dyadic ties between these actors." 
  "Réseaux sociaux" "Un réseau social est un ensemble d'entités, telles que des individus ou des organisations, reliées entre elles par des liens créés lors d'interactions sociales."
  "Social structure")
(defc "Social structure" "..." "Structure sociale" "..." 
  "Root")
(defc "Software" "Computer software, or simply software is any set of machine-readable instructions that directs a computer's processor to perform specific operations"
  "Logiciel" "En informatique, un logiciel est un ensemble de séquences d’instructions interprétables par une machine et d’un jeu de données nécessaire à ces opérations"
  "Root")
(defc "Software method" "..." "Méthode logicielle" "..."
  "Method")
(defc "Software Services" "In the context of enterprise architecture, service-orientation and service-oriented architecture, the term service refers to a set of related software functionalities that can be reused for different purposes, together with the policies that should control its usage" 
  "Service logiciel" "Dans le contexte de l'entreprise un service logiciel fait référence à un ensemble de fonctionnalités qui peuvent être réutilisées dans des contextes différents, accompagnées de règles contrôlant leur usage."
  "Software")
(defc "Software Tool" "A programming tool or software development tool is a computer program that software developers use to create, debug, maintain, or otherwise support other programs and applications" 
  "Outil logiciel" "Un outil de développement logiciel est unprogramme utilisé par les concepteurs de logiciel pour créer, debugger et maintenir d'autres programmes ou applications."
  "Software")
(defc "Theory" "Theory is a contemplative and rational type of abstract or generalizing thinking, or the results of such thinking. Depending on the context, the results might for example include generalized explanations of how nature works." 
  "Théorie" "Une théorie est un ensemble d'explications, de notions ou d'idées sur un sujet précis, pouvant inclure des lois et des hypothèses, induites par l'accumulation de faits trouvés par l'observation ou l'expérience"
  "Concept")
(defc "Transportation" "Transport or transportation is the movement of people, animals and goods from one location to another. Modes of transport include air, rail, road, water, cable, pipeline and space. The field can be divided into infrastructure, vehicles and operations." 
  "Transport" "Le transport désigne le type de déplacement d'objets, de marchandises, ou d'individus (humains ou animaux) d'un endroit à un autre. Ces modes de transport incluent l'aviation, le chemin de fer, le transport routier, le transport maritime, le transport par câble, l'acheminement par pipe-line et le transport spatial."
  "Industry")
(defc "Collaborative Work, Peer Production" "Peer production (also known as mass collaboration) is a way of producing goods and services that relies on self-organizing communities of individuals. In such communities, the labor of a large number of people is coordinated towards a shared outcome."
  "Travail collabortif" "Le travail est souvent naturellement collectif et collaboratif, c'est-à-dire qu'il fait interagir plusieurs acteurs pour la réalisation de tâches qui visent à atteindre un but commun. La notion de travail collaboratif désigne aujourd'hui un travail qui n'est plus fondé sur l'organisation hiérarchisée traditionnelle, et plus spécifiquement un nouveau mode de travail (éventuellement intégré dans un modèle économique de production) où collaborent de nombreuses personnes grâce aux technologies de l'information et de la communication."
  "Method")
(defc "Virtual Environment" "Virtual environment software refers to any software, program or system that implements, manages and controls multiple virtual environment instances"
  "Environnement virtuel" "La virtualisation consiste à faire fonctionner un ou plusieurs systèmes d'exploitation1 / applications comme un simple logiciel, sur un ou plusieurs ordinateurs2 - serveurs / système d'exploitation, au lieu de ne pouvoir en installer qu'un seul par machine."
  "Computer tool")
(defc "Web Service" "A Web Service is a method of communication between two electronic devices over a network. It is a software function provided at a network address over the web with the service always on as in the concept of utility computing" 
  "Service Web" "Un service web est un programme informatique de la famille des technologies web permettant la communication et l'échange de données entre applications et systèmes hétérogènes dans des environnements distribués"
  "Software")
(defc "Wireless Sensor" "..." "Capteur sans fil" "..."
  "Hardware")
(defc "Wireless Sensor Network" "A wireless sensor network (WSN) of spatially distributed autonomous sensors to monitor physical or environmental conditions, such as temperature, sound, pressure, etc. and to cooperatively pass their data through the network to a main location"
   "Réseau de capteurs sans fil" "Un réseau de capteurs sans fil est un réseau ad hoc avec un grand nombre de nœuds qui sont des micro-capteurs capables de récolter et de transmettre des données environnementales d'une manière autonome"
   "Computer network")
(defc "Workflow" "A workflow consists of an orchestrated and repeatable pattern of business activity enabled by the systematic organization of resources into processes that transform materials, provide services, or process information."
  "Workflow, Flux de travaux" "Un workflow, anglicisme pour flux de travaux, est la représentation d'une suite de tâches ou opérations effectuées par une personne, un groupe de personnes, un organisme, etc."
  "Method")
(defc "Workspace" "..." "Espace de travail" "..."
  "Concept")
           
;;;========================================================================= ROLES

(defindividual "Role Type"
    ("label" 
     ((:en "includes")
      (:fr "comprend" "est composé de")))
  ("definition" 
   ((:en "Includes means that the target concept is part of the initial concept.")
    (:fr "Comprend signifie que les concepts cibles font partie du concept de départ.")))
  ("author" "barthès")
  ("from" $e-cscwd-concept.11)
  ("target" $e-cscwd-concept.12)
  ("creation date" "21/08/2014")
  )

;;;=============================================================================== 
(format t "End loading CSCWD-ONTO-ONTOLOGY.lisp")

:EOF
