;;;==============================================================================
;;;14/09/30
;;;                                UTILS.lisp
;;;
;;;==============================================================================

;;; Set of utility functions for rendering, marshalling/unmarshalling MOSS objects


#|
2014
 0930 Creation (mfk)
|#

;; ============================================================================
;; Function: marshall-moss-object
;; Creates an alist representation of an object or a list of objects
;; ============================================================================

(defun marshall-moss-object (elem &optional inv-rel-to-expand &optional expanded-list)
  (when (not expanded-list)
    (push nil expanded-list))
  (let (result-rep obj-rep to-expand uid summ)
    (cond 
      ;; if elem is a list (presumably moss objects!)
      ((listp elem)
        (dolist (e elem)
          (let ((response (marshall-moss-object e inv-rel-to-expand expanded-list)))
            (when response
              (setq result-rep (append result-rep response))))
        )
        ;; identify direct and indirect objects
        (setq result-rep (mapcar #'(lambda(x)  
            (let (moss-id)
              (setq moss-id (cadr (assoc "marshaller::moss-id" (cdr x) :test #'equal)))
              (if (find moss-id elem)
                (append x (list (list "marshaller::condition" :direct)))
                (append x (list (list "marshaller::condition" :indirect))))               
            )
          ) result-rep))
      )
      ;; if elem is a moss object
      ((moss::%pdm? elem)
        ;; if elem is already in the expanded list, ignore the expansion
        (when (find elem expanded-list)
          (return-from marshall-moss-object))
        ;; add type to object
        (push (car (moss::%get-object-class-name elem)) obj-rep)
        ;; look for a unique entry point
        (setq uid (moss::%get-entry-point-if-unique elem))
        ;; if there is no uid, use moss id (not it will not be used by the unmarshaller, 
        ;; only used as a guide!)
        (if (not uid)
          (setq uid (car (moss::%get-value elem 'moss::$id)))
          (setq uid (symbol-name uid))
        )
        (push (list "marshaller::uid" uid) obj-rep)
        (push (list "marshaller::moss-id" (car (moss::%get-value elem 'moss::$id))) obj-rep)
        ;; get summary
        (setq summ (car (send elem '=summary)))
        (if (stringp summ)
          (push (list "marshaller::summary" summ) obj-rep)
          (push (list "marshaller::summary" (symbol-name summ)) obj-rep)
        )        
        ;; iterate over properties
        (let (prop-name)
          (dolist (property (moss::%get-properties elem))
            ;; get property name
            (setq prop-name (car (moss::%%get-value property 'moss::$pnam *context*)))
            ;; extract canonical name
            (let (partial-rep)
              (push (mln::get-canonical-name prop-name) partial-rep)    
              (cond
                ((moss::%is-attribute? property)
                    ;; iterate over values
                    ;;(dolist (item (moss::%get-value elem property))
                    (dolist (item (send elem '=get (mln::get-canonical-name prop-name)))
                      (push item partial-rep)))
                ((moss::%is-relation? property)
                    ;; iterate over references
                    ;;(dolist (item (moss::%get-value elem property))
                    (dolist (item (send elem '=get (mln::get-canonical-name prop-name)))
                      ;; look for a unique entry point of object
                      (setq uid (moss::%get-entry-point-if-unique item))
                      ;; if there is no uid, use moss id (not it will not be used by the unmarshaller, only used as a guide!)
                      (if (not uid)
                        (setq uid (car (moss::%get-value item 'moss::$id)))
                        (setq uid (symbol-name uid))
                      )
                      (push uid partial-rep)
                      ;; if reference is not in the expanded list, so it has to be expanded!
                      (when (not (find item expanded-list))
                        (push item to-expand))))
                (t
                  (error "not an attribute nor a relation")))
              ;; add attribute only if it has values
              (when (> (length partial-rep) 1)
                (push (reverse partial-rep) obj-rep))))) 
        ;; iterate over inverse properties
        (when inv-rel-to-expand
          (format t "~%@marshall-moss-object - entered inverse for [~S] - inv-rel-to-expand: [~S]" elem inv-rel-to-expand)                  
          (dolist (inv-rel 
                    (moss::%%get-all-class-inverse-relations 
                      (moss::%%get-id 
                        (car (moss::%get-object-class-name elem)) :class)))
            (let (src-prop src-prop-name src-canonical-name partial-rep)                          
              (format t "~%@marshall-moss-object - each inv-rel for [~S] - inv-rel: [~S]" elem inv-rel)            
              ;; get source property
              (setq src-prop (car (moss::%%get-value inv-rel 'moss::$inv.of *context*)))
              ;; if property is marked to expand
              (when (find src-prop inv-rel-to-expand)
                (format t "~%@marshall-moss-object - each inv-rel for [~S] - src-prop [~S] is desired" elem src-prop)            
                ;; get src property name
                (setq src-prop-name (car (moss::%%get-value src-prop 'moss::$pnam *context*)))              
                ;; get canonical name
                (setq src-canonical-name (mln::get-canonical-name src-prop-name))
                ;; add prefix
                (setq src-canonical-name (string+ "inverse::" src-canonical-name))
                ;; iterate over references
                (push src-canonical-name partial-rep)
                (dolist (item (moss::%get-value elem inv-rel))
                  ;; look for a unique entry point of object
                  (setq uid (moss::%get-entry-point-if-unique item))
                  ;; if there is no uid, use moss id (not it will not be used by the unmarshaller, only used as a guide!)
                  (if (not uid)
                    (setq uid (car (moss::%get-value item 'moss::$id)))
                    (setq uid (string+ "&" (symbol-name uid)))
                  )                  
                  (push uid partial-rep)
                  ;; if reference is not in the expanded list, so it has to be expanded!
                  (when (not (find item expanded-list))
                    (push item to-expand)
                  )
                )
                ;; add partial representation to object representation
                (when (> (length partial-rep) 1)
                  (push (reverse partial-rep) obj-rep)
                )                
              )
            )
          )
        )
        ;; add the object to the result representation
        (push (reverse obj-rep) result-rep)        
        ;; add object to the expanded-list
        (setf (cdr (last expanded-list)) (cons elem nil))        
        ;; process to-expand list
        (dolist (te to-expand)
          (let ((result (marshall-moss-object te inv-rel-to-expand expanded-list)))
            (when result
              (setq result-rep (append result-rep result))))))
      ;; not an object nor a list
      (t
        (error "element is not an object nor a list")))    
    (return-from marshall-moss-object (reverse result-rep))))
    
#|
(defconcept "credit-card6"
	(:att "number")
	(:att "name"))
	
(defconcept "type6"
	(:att "acronym")
	(:att "name")
	(:rel "cards" (:to "credit-card6")))

(defrelation "type" (:from "credit-card6") (:to "type6"))

(defindividual "credit-card6" ("number" "123") ("name" "Marcio Fuckner"))
(defindividual "credit-card6" ("number" "456") ("name" "Ana Maria"))
(defindividual "credit-card6" ("number" "789") ("name" "Giovani Bernardes"))

(defindividual "type6" ("acronym" "VISA") ("name" "VISA CORP") ("cards" $e-credit-card6.1 $e-credit-card6.3))
(defindividual "type6" ("acronym" "AMEX") ("name" "American Express") ("cards" $e-credit-card6.2))

(send '$e-credit-card6.1 '=add-attribute-values 'has-type '$e-type6.1)
(send '$e-credit-card6.2 '=add-attribute-values 'has-type '$e-type6.2)
(send '$e-credit-card6.3 '=add-attribute-values 'has-type '$e-type6.1)

(marshall-moss-object '($e-credit-card6.1 $e-credit-card6.2 $e-credit-card6.3))

|#

#|
(defconcept "credit-card8"
	(:att "number" (:entry) (:unique))
	(:att "name"))
	
(defconcept "type8"
	(:att "acronym" (:entry) (:unique))
	(:att "name")
	(:rel "cards" (:to "credit-card8")))

(defrelation "type" (:from "credit-card8") (:to "type8"))

(defindividual "credit-card8" ("number" "123") ("name" "Marcio Fuckner"))
(defindividual "credit-card8" ("number" "456") ("name" "Ana Maria"))
(defindividual "credit-card8" ("number" "789") ("name" "Giovani Bernardes"))

(defindividual "type8" ("acronym" "VISA") ("name" "VISA CORP") ("cards" $e-credit-card8.1 $e-credit-card8.3))
(defindividual "type8" ("acronym" "AMEX") ("name" "American Express") ("cards" $e-credit-card8.2))

(send '$e-credit-card8.1 '=add-attribute-values 'has-type '$e-type8.1)
(send '$e-credit-card8.2 '=add-attribute-values 'has-type '$e-type8.2)
(send '$e-credit-card8.3 '=add-attribute-values 'has-type '$e-type8.1)

(marshall-moss-object '($e-credit-card8.1 $e-credit-card8.2 $e-credit-card8.3))

|#

(defun unmarshall-moss-object (elem)
  (let (concept-name original-individual-uid new-individual-uid prop-name conversion-table condition-list)
    ;; iterate over the elements ignoring the relationships for the moment
    (dolist (e elem)
      (setq concept-name (car e))
      (setq original-individual-uid (cadadr e))
      (setq new-individual-uid nil)
      ;; if id is a string then look for an object related to this "possible" entry point
      (when (stringp original-individual-uid)
        (setq new-individual-uid (car (moss::find-objects-from-text original-individual-uid :target concept-name))))
      ;; create a new individual if needed
      (when (not new-individual-uid)
        (setq new-individual-uid (eval `(defindividual ,concept-name))))
      ;; update converstion table adding old id and new id
      (push (list original-individual-uid new-individual-uid) conversion-table)      
      ;; work with attributes using add addtribute values and remove-attribute-values
      (let (work-list property)
        (dolist (prop (cddr e))
          (setq prop-name (car prop))
          (setq property (moss::%%get-id prop-name :attribute :class-ref concept-name))
          ;; only attribute from now!
          (when property
            (push (cons prop-name (cdr prop)) work-list)))
        (setq work-list (reverse work-list))
        ;; updating attributes
        (dolist (elem work-list)
          (send new-individual-uid '=replace (car elem) (cdr elem)))))
    ;; iterate over the elements - rebuilding relationships
    (dolist (e elem)
      ;; recover concept name, original and new uid
      (setq concept-name (car e))
      (setq original-individual-uid (cadadr e))
      (setq new-individual-uid (cadr (assoc original-individual-uid conversion-table :test #'equal)))
      ;; now it's time to rebuild relationships
      (let (work-list property work-prop)
        ;; iterate over properties to find relationships
        (dolist (prop (cddr e))
          (setq prop-name (car prop))
          (setq property (moss::%%get-id prop-name :relation :class-ref concept-name))
          (when property
            (push prop-name work-prop)
            ;; iterate over each property value
            (dolist (v (cdr prop))
              (setq corresp-ref (assoc v conversion-table :test #'equal))
              (if corresp-ref
                (push (cadr corresp-ref) work-prop)
                (error "unmarshalling error - relationship error")))
            (push (reverse work-prop) work-list)))
        (setq work-list (reverse work-list))
        ;; updating relations
        (dolist (elem work-list)
          (send new-individual-uid '=replace (car elem) (cdr elem)))))    
    ;; retrieve object conditions
    (setq condition-list 
      (mapcar #'(lambda(x) 
                  (list (cadr (assoc "marshaller::uid" (cdr x) :test #'equal)) 
                        (cadr (assoc "marshaller::condition" (cdr x) :test #'equal)))) elem
      )
    )
    (mapcar #'(lambda(x)
                (let (condition)
                  (setq condition (cadr (assoc (car x) condition-list)))
                  (list (cadr x) condition)
                )
              ) conversion-table
    )
  )
)

#|
(unmarshall-moss-object 
'(
 ("credit-card7" ("moss::$id" $E-CREDIT-CARD7.100) ("number" "456") ("name" "Ana Maria") ("type" $E-TYPE7.100))
 ("type7" ("moss::$id" $E-TYPE7.100) ("acronym" "VISA") ("name" "VISA CORP") ("cards" $E-CREDIT-CARD7.100))
 ))
 
 (unmarshall-moss-object 
'(
  ("credit-card8" ("uid" "999") ("number" "999") ("name" "Ana Maria") ("type" "AMEIXA"))
  ("type8" ("uid" "AMEIXA") ("acronym" "AMEIXA") ("name" "American Express") ("cards" "999"))
 ))
 |#
 
;; --------------------------------------------------------------------------
;; Transform a moss alist representation into a json format
;; --------------------------------------------------------------------------

(defun moss-list-to-json (result &optional language &optional pred-ref )
  "Creates a JSON representation of a MOSS list of individuals
Arguments:
   result: the resulting moss list
   pred-ref (optional): the reference of the predecessor. 
Return:
   the list in a json format"
  (setf language (or language *language*))
  (let (json card prop-ref list-flag)
    (cond
     ;; is an attribute representation
     ((is-attr-rep result)            
      ;; check attribute cardinality
      (setq list-flag t)
      (setq prop-ref nil)
      (setq card nil)
      (when (not (equal nil pred-ref))
        (setq prop-ref (moss::%%get-id-for-property (car result) :property :class-ref pred-ref))
        (setq card (car (moss::%get-value prop-ref 'moss::$maxt)))
      )
      ;; avoid list creation if we have <= one element with cardinality equal to 1
      (when (and (<= (length (cdr result)) 1) (equal card 1))
          (setq list-flag nil))
      
      (setq json (concatenate 'string "\"" (car result) "\"" " : "))
      (when list-flag
        (setq json (concatenate 'string json " [ "))
      )
      (let ((index 0))
        (mapc #'(lambda(x) 
                  (setq json (concatenate 'string json (eval-attr-value pred-ref (car result) x language) ))
                  (incf index)
                  (if (< index (list-length (cdr result)))
                      (setq json (concatenate 'string json " , "))))
          (cdr result)))
      (when list-flag
        (setq json (concatenate 'string json " ] ")))
     
     ;; if empty and without list-flag
     (when (and (equal (length result) 1) (not list-flag))
       (setq json (concatenate 'string json "null"))))
      
     ;; is an object representation
     ((is-obj-rep result)
      (setq json (concatenate 'string json " { "))
      (setq json (concatenate 'string json "\"marshaller::type\" : \"" 
                   (car result) "\","))
      (let ((index 0))
        (mapc #'(lambda(x)
                  (setq json (concatenate 'string 
                               json (moss-list-to-json x language (car result) ))) 
                  (incf index)
                  (if (< index (list-length (cdr result)))
                      (setq json (concatenate 'string json " , ")))) 
          (cdr result)))
      (setq json (concatenate 'string json " } ")))
     ;; is a list of objects
     ((is-list-obj-rep result)
      (setq json (concatenate 'string json " [ "))
      (let ((index 0))
        (mapc #'(lambda(x)
                  (setq json (concatenate 'string 
                               json (moss-list-to-json x language))) 
                  (incf index)
                  (if (< index (list-length result))
                      (setq json (concatenate 'string json " , ")))) result))
      (setq json (concatenate 'string json " ] "))))	
    json))

;; --------------------------------------------------------------------------
;; Check of symbol is a list of moss objects (alist format)
;; --------------------------------------------------------------------------
(defun is-list-obj-rep (ref-symbol)
  "check if the symbol is a list of moss objects
Arguments:
   ref-symbol: the moss list
Return:
   a boolean result"
  ;; must be a list
  (if (not (listp ref-symbol))
      (return-from is-list-obj-rep nil))
  ;; evaluate each list member to check its contents	
  (mapc #'(lambda(x) 
            (if (not (is-obj-rep x))
                (return-from is-list-obj-rep nil))) 
    (cdr ref-symbol))
  t)		 

  ;; --------------------------------------------------------------------------
;; Check of symbol is a moss object (alist format)
;; --------------------------------------------------------------------------

(defun is-obj-rep (ref-symbol)
  "check if the symbol is according to the moss object format
Arguments:
   ref-symbol: the moss potential object
Return:
   a boolean result"
  ;; must be a list
  (if (not (listp ref-symbol))
      (return-from is-obj-rep nil))
  ;; to be an object the first element must be a string
  (if (not (stringp (car ref-symbol)))
      (return-from is-obj-rep nil))
  ;; to be an object the remainder must be attributes
  (mapc #'(lambda(x) 
            (if (not (is-attr-rep x))
                (return-from is-obj-rep nil))) 
    (cdr ref-symbol))
  t)

;; --------------------------------------------------------------------------
;; Check of symbol is a moss attribute (alist format)
;; --------------------------------------------------------------------------

(defun is-attr-rep (ref-symbol)
  "check if the symbol is according to the moss object format
Arguments:
   ref-symbol: the moss potential object
Return:
   a boolean result"
  (if (not (listp ref-symbol))
      (return-from is-attr-rep nil))
  ;; to be an attribute the first element must be a string
  (if (not (stringp (car ref-symbol)))
      (return-from is-attr-rep nil))
  ;; to be an attribute the remainder must be a:
  ;; string, symbol, mln string, an object
  (mapc #'(lambda(x) 
            (if (and (not (stringp x) )
                     (not (symbolp x) )
                     (not (mln::mln? x))
                     (not (is-obj-rep x)))
                (return-from is-attr-rep nil))) 
    (cdr ref-symbol))
  t)	  

;; --------------------------------------------------------------------------
;; Evaluate attribute during transformation
;; --------------------------------------------------------------------------

(defun eval-attr-value (class-ref attr-name attr-value language)
  "this function deals with a variety of attribute types (mln,ids,references)
Arguments:
   class-ref: the class reference
   attr-name: the attribute name
   attr-value: the attribute
Return:
   changed attribute"
  (cond
    ;; if the content is multilingual
    ((mln::mln? attr-value)
      ;; extracts the corresponding value according to the language 
      (return-from eval-attr-value 
        (concatenate 'string "\"" (mln::print-string attr-value language) "\""))
    )
    ;; if the attribute is a unique id
    ((equal attr-name "marshaller::uid")
      ;; if the content is a string then concatenate the symbol & at the first position
      (when (stringp attr-value)
        (return-from eval-attr-value
          (concatenate 'string "\"&" attr-value "\"")))
      ;; is a symbol then create the string representation
      (when (symbolp attr-value)
        (return-from eval-attr-value
          (concatenate 'string "\"" (symbol-name attr-value) "\""))
      )
    )
    ;; a symbol value as a content is presumably a relationship (we don't need to check)
    ((symbolp attr-value)
      (return-from eval-attr-value
        (concatenate 'string "\"" (symbol-name attr-value) "\""))
    )
    ;; a string value with related class (must investigate if it is a relation
    ((and class-ref (stringp attr-value))
      (if (moss::%%get-id attr-name :relation :class-ref class-ref)
         ;; add an address symbol in front of the string if is a reference    
        (return-from eval-attr-value (concatenate 'string "\"&" attr-value "\""))
        (return-from eval-attr-value (concatenate 'string "\"" attr-value "\""))
      )
    )
    ;; a normal string value
    ((stringp attr-value)
      (return-from eval-attr-value (concatenate 'string "\"" attr-value "\""))
    )
    ;; a complex attribute (list)
    ((listp attr-value)
      (return-from eval-attr-value (moss-list-to-json attr-value language))
    )    
  )
)

:EOF