﻿;;;-*- Mode: Lisp; Package: "CIT-FAC-3" -*-
;;;===============================================================================
;;;10/05/19
;;;                                AGENT CIT-FAC-3 
;;;
;;;===============================================================================

#|
2009
 0606 prefixing agent names with CIT-
 0730 fixing environment in skills
2010
 0519 updating to version 8 
|#

;;; define name in the loading environment, so that it can be used without prefix,
;;; export it to the agent package

(defpackage :CIT-FAC-3 (:use :moss :omas :cl #+MCL :ccl))
(in-package :CIT-FAC-3)

(omas::defagent :CIT-FAC-3 :redefine t)

;;; =============================== skill section ==============================
;;;  globals  

(defParameter *multiply-timeout* 4 "global default timeout for the multiply subtasks")

;;;===============================================================================

(omas::defskill :dumb-fac :CIT-FAC-3
  :static-fcn static-dumb-fac
  :dynamic-fcn dynamic-dumb-fac
  :time-limit-fcn time-limit-dumb-fac
  )

(defun static-dumb-fac (agent message nn)
  (declare (ignore message))
  ;; if nn is less than or equal to 1, then return 1 immediately
  (if (< nn 2) (static-exit agent 1)
      ;; otherwise, create a subtask for computing the product of the first 
      ;; two top values
      (let ()
        ;; ship subtask to agent CIT-MUL-5 to compute
        (send-subtask agent :to :CIT-MUL-5 :action :multiply 
                      :args (list nn (1- nn)))
        ;; define a tag (:n) in the environment to record the value of the next
        ;; products to compute. e.g., nn-2 -> (nn - 2)!
        (env-set agent (- nn 2) :n)
        ;; quit
        (static-exit agent :done))))

(defun dynamic-dumb-fac (agent message answer)
  "this function is called whenever we get a result from a subtask. This ~
   approach is not particularly clever, since the computation is linear ~
   and uses the same multiplying agent, i.e., CIT-MUL-5."
   (declare (ignore message))
  (let ((nn (env-get agent :n)))
    ;; if the recorded value is 1 or less, then we are through
    (if (< nn 2)
      ;; thus we do a final exit (in fact we should never go there ??
      (dynamic-exit agent answer)
      ;; otherwise we multiply the answer with the next high number
      ;; creating a subtask
      (progn
        (send-subtask agent :to :CIT-MUL-5 :action :multiply
                      :args (list answer nn))
        ;; update environment
        (env-set agent (1- nn) :n)
        ;; then return an answer
        answer
        ))))

(defun time-limit-dumb-fac (agent dummy message)
  "when reaching a time-limit for computing the function we simply abort all ~
   subtasks and quit."
  (declare (ignore message dummy))
  ;; cancel current subtasks
  (cancel-all-subtasks agent)
  ;; and return with :error flag
  (dynamic-exit agent :error))

;;; =============================== skill section ==============================

(defskill :greedy-fac :CIT-FAC-3
  :static-fcn static-greedy-fac
  :dynamic-fcn dynamic-greedy-fac
  :timeout-handler timeout-greedy-fac
  )

(defun static-greedy-fac (agent message nn)
  "Here we assume that we have 3 acquaintances that know how to multiply and ~
   we try to use them as best as we can. I.e., we distribute the first three ~
   multiplications, then each time an answer comes back we send a new multiplication ~
   to the agent that perfomed the subtask."
  (declare (ignore message))
  ;; if nn is less than or equal to 1, then return 1 immediately
  (if (< nn 2) (static-exit agent 1)
      ;; otherwise, create a series of subtasks for computing the product of the first 
      ;; top values
      ;; we initialize a :res value in the environment (used by dynamic function)
      (let ()
        (env-set agent 1 :res)
        ;; ship subtask to agent CIT-MUL-5 to compute
        (send-subtask agent :to :CIT-MUL-5 :action :multiply 
                      :args (list nn (1- nn)) :timeout *multiply-timeout*)
        ;; decrease nn
        (decf nn 2)
        ;; when nn is greater or equal than 2 continue
        (when (> nn 3)
          (send-subtask agent :to :CIT-MUL-6 :action :multiply 
                        :args (list nn (1- nn)) :timeout *multiply-timeout*)
          ;; decrease nn
          (decf nn 2))
        ;; when nn is greater or equal than 2 continue
        (when (> nn 3)
          (send-subtask agent :to :CIT-MUL-5 :action :multiply 
                        :args (list nn (1- nn)):timeout *multiply-timeout*)
          (decf nn 2))
        ;; define a tag (:n) in the environment to record the value of the next
        ;; products to compute
        (env-set agent nn :n)
        ;; quit
        (static-exit agent :done))))

(defun dynamic-greedy-fac (agent message answer)
  "this function is called whenever we get a result from a subtask. "
   (declare (ignore message))
  (let ((nn (env-get agent :n))
        (res (env-get agent :res))
        (list-of-subtasks (pending-subtasks? agent)))
    (format t "~&Dynamic: nn=~s - (pending-subtasks? agent) ~S"
            (env-get agent :res )(pending-subtasks? agent))
    ;; we are finished when this was the last substasks and we do not have
    ;; partial result waiting (res=1)
    (cond 
     ((and (< nn 2) (null (cdr list-of-subtasks))(eql res 1))
      ;; thus we do a final exit, we just received the final answer
      (dynamic-exit agent answer))
     ;; if the recorded value is greater than 1, then we send a new subtask to the
     ;; agent that sent the answer
     ((> nn 1)
      (send-subtask agent :to (answering-agent agent) :action :multiply
                    :args (list answer nn) :timeout *multiply-timeout*)
      ;; update environment
      (env-set agent (1- nn) :n)
      ;; then return an answer (actually to nobody in particular)
      answer)
     ;; otherwise we exhausted primary values, we must gather results of MUL
     ;; agents and multiply them together
     ((eql 1 res)
      ;; if 1 then we store partial result for next time around
      (env-set agent answer :res)
      ;; then return an answer (actually to nobody in particular)
      answer)
     ;; otherwise we have a partial result in the environment
     ((not (eql 1 res))
      ;; use it to send a new multiply job
      (send-subtask agent :to (answering-agent agent) :action :multiply
                    :args (list answer (env-get agent :res))
                    :timeout *multiply-timeout*)
      ;; reset environment
      (env-set agent 1 :res)
      ;; then return an answer (actually to nobody in particular)
      answer)
     ;; otherwise we are in trouble (something wrong with the algorithm)
     (t (error "bad greedy fac algorithm")))))

(defun timeout-greedy-fac (agent message)
  "function for handling timeout errors"
  (let (dead-agent new-choice)
    (case (omas::action message)
      (multiply
       ;; if a multiply agent is timed out, then reallocate to another one
       ;; first get the name of the agent who was suppose to do the job
       (setq dead-agent (omas::to! message))
       ;; pick up another agent from the list of acquaintances
       (setq new-choice 
             ((lambda (xx)(nth (random (1- (length xx)))
                               (remove (omas::name dead-agent) xx)))
              '(:CIT-MUL-5 :CIT-MUL-6 :?EC-MUL-3)))
       (format t "~&dead agent: ~S - new-choice: ~S" 
               (omas::name dead-agent) new-choice)
       ;; reallocate the work
       (send-subtask agent :to new-choice :action :multiply
                     :args (omas::args message)
                     :timeout *multiply-timeout*)
       ;; exit
       :done
       )
      (otherwise 
       ;; for any other skill we let the system process the timeout condition
       :unprocessed))
    ))

; (trace (timeout-greedy-fac :step t))
; (untrace timeout-greedy-fac)

;;; =============================== skill section ==============================

(defskill :fast-fac :CIT-FAC-3
  :static-fcn static-fast-fac
  :dynamic-fcn dynamic-fast-fac)

(defun static-fast-fac (agent message nn)
  "this skill works by first creating n/2 tasks for distribution to the ~
   multiplying agents, i.e., (* n n-1) (* n-2 n-3) ...
  if n is even we end up with (* 2 1)
   if n is odd we end up with (° 3 2)
  It then broadcasts those tasks, record the left over number (1 or 2) and quits."
  (declare (ignore message))
  (let ((*default-call-for-bids-timeout-delay* 3))
    ;; we want to control the call for bid delay for the contract net to prevent early
    ;; aborts
    (if (< nn 2) (static-exit agent 1)
        ;; this line should be replaced with (if (< nn 3) (return (max nn 1))
        ;; to avoid subcontracting (* 2 1)
        (let (env (delay 0))
          ;; try to produce as many initial subtasks as possible
          (loop
            ;; create n/2 tasks for distribution to the multiplying agents
            ;; create a subtask for computing the product of the next two values
            ;; broadcast
            (send-subtask agent :to :ALL :action :multiply 
                          :args (list nn (decf nn)) :delay delay 
                          :protocol :contract-net)
            (decf nn) ; adjust nn for next round
            (incf *default-call-for-bids-timeout-delay* 3) ; increase waiting time
            ;;... to let agents make their bids
            (incf delay) ; we assume 1 as cost of setting up subtask
            ;; if nn becomes less than 3 don't multiply 2 by 1! get out of the loop
            (if (< nn 3)(return nil)))
          ;; define the :n tag in the environment to record the value of the next
          ;; products to compute. When exiting the loop it can be 2, 1, or 0. We set
          ;; it to 2 or 1
		  (env-set agent (if (< nn 2) 1 nn) :res)
          ;; quit
          (static-exit agent :done)))))

(defun dynamic-fast-fac (agent message answer)
  "this function is called whenever we get a result from a subtask.
   We structured the environment as follow:
     :res contains a partial result (nil or 2 at the beginning)
   We use :res as follows:
     if 1 we save the result into it
     otherwise, we broadcast the multiplication of :res with the result."
  (declare (ignore message))
  (let (res)
    ;(format t "~&Dynamic: nn=~s - (pending-subtasks?) ~S"
    ;        (cadr (member :res environment)) (pending-subtasks? agent))
    ;; we are finished when there are no more active substasks.
    (cond 
     ((not (pending-subtasks? agent))
      ;; thus we do a final exit
      (dynamic-exit agent answer))
     ((eql 1 (setq res (env-get agent :res)))
      ;; we have still subtasks in progress and res is nil
      ;; store the answer
	  (env-set agent answer :res)
      ;; check if this was the last subtask in the list
      (if (null (cdr (pending-subtasks? agent)))
        (return-from dynamic-fast-fac (dynamic-exit agent answer)))
      ;; otherwise return answer
      answer)
     (t
      ;; res is not 1, i.e., it contains a partial result
      ;; we multiply the answer with res creating a subtask
      ;; ... broadcasting it
      (send-subtask agent :to :ALL :action :multiply
                    :args (list answer res) :protocol :contract-net)
      ;; update environment
	  (env-set agent 1 :res)
      answer))))

;;; =============================== skill section ==============================
;;; another strategy for computing fac using contract-net

(defskill :fast-fac-with-timeout :CIT-FAC-3
  :static-fcn static-fast-fac-with-timeout
  :dynamic-fcn dynamic-fast-fac-with-timeout
  :timeout-handler timeout-fast-fac-with-timeout)

(defun static-fast-fac-with-timeout (agent message nn)
  "this skill works by first creating n/2 tasks for distribution to the ~
   multiplying agents, i.e., (* n n-1) (* n-2 n-3) ...
  if n is even we end up with (* 2 1)
   if n is odd we end up with (° 3 2)
  It then broadcasts those tasks, record the left over number (1 or 2) and quits."
  (declare (ignore message))
  (let ((*default-call-for-bids-timeout-delay* 3))
    ;; we want to control the call for bid delay for the contract net to prevent early
    ;; aborts
    (if (< nn 2) (static-exit agent 1)
        ;; this line should be replaced with (if (< nn 3) (return (max nn 1))
        ;; to avoid subcontracting (* 2 1)
        (let (env (delay 0))
          ;; try to produce as many initial subtasks as possible
          (loop
            ;; create n/2 tasks for distribution to the multiplying agents
            ;; create a subtask for computing the product of the next two values
            ;; broadcast
            (send-subtask agent :to :ALL :action :multiply 
                          :args (list nn (decf nn)) :delay delay 
                          :protocol :contract-net :timeout *multiply-timeout*)
            (decf nn) ; adjust nn for next round
            (incf *default-call-for-bids-timeout-delay* 3) ; increase waiting time
            ;;... to let agents make their bids
            (incf delay) ; we assume 1 as cost of setting up subtask
            ;; if nn becomes less than 3 don't multiply 2 by 1! get out of the loop
            (if (< nn 3)(return nil)))
          ;; define the :n tag in the environment to record the value of the next
          ;; products to compute. When exiting the loop it can be 2, 1, or 0. We set
          ;; it to 2 or 1
		  (env-set agent (if (< nn 2) 1 nn) :res)
          ;; quit
          (static-exit agent :done)))))

(defun dynamic-fast-fac-with-timeout (agent message answer)
  "this function is called whenever we get a result from a subtask.
   We structured the message as follow:
     :res contains a partial result (nil or 2 at the beginning)
   We use :res as follows:
     if 1 we save the result into it
     otherwise, we broadcast the multiplication of :res with the result."
	 (declare (ignore message))
  (let (res)
    ;(format t "~&Dynamic: nn=~s - (pending-subtasks?) ~S"
    ;        (env-get agent :res) (pending-subtasks? agent))
    ;; we are finished when there are no more active substasks.
    (cond 
     ((not (pending-subtasks? agent))
      ;; thus we do a final exit
      (dynamic-exit agent answer))
     ((eql 1 (setq res (env-get agent :res)))
      ;; we have still subtasks in progress and res is nil
      ;; store the answer
	  (env-set agent answer :res)
      ;; check if this was the last subtask in the list
      (if (null (cdr (pending-subtasks? agent)))
        (return-from dynamic-fast-fac-with-timeout 
          (dynamic-exit agent answer)))
      ;; otherwise return answer
      answer)
     (t
      ;; res is not 1, i.e., it contains a partial result
      ;; we multiply the answer with res creating a subtask
      ;; ... broadcasting it
      (send-subtask agent :to :ALL :action :multiply
                    :args (list answer res) :protocol :contract-net
                    :timeout *multiply-timeout*)
      ;; update environment
	  (env-set agent 1 :res)
      answer))))

(defun timeout-fast-fac-with-timeout (agent message)
  "function for handling timeout errors"
  (case (omas::action message)
    (multiply
     ;; if a multiply agent is timed out, then reallocate to another one
     ;; by simply doing another broadcast
     (send-subtask agent :to :ALL :action :multiply
                   :args (omas::args message)
                   :protocol :contract-net
                   :timeout *multiply-timeout*)
     ;; exit
     :done
     )
    (otherwise 
     ;; for any other skill we let the system process the timeout condition
     :unprocessed))
  )

;;; =============================== skill section ==============================


;;;  parameters

(defParameter *multiply-timeout 4)
(defParameter *timeout-delay* 3)

;;; using contract-net with limited number of pending subtasks

(defskill :limited-fac :CIT-FAC-3
  :static-fcn static-limited-fac
  :dynamic-fcn dynamic-limited-fac
  :timeout-handler timeout-limited-fac
  )

(defun static-limited-fac (agent message nn)
  "Here we send a limited number of contracts, then each time an answer comes ~
   back we send a new contract."
  (declare (ignore message))
  ;; if nn is less than or equal to 1, then return 1 immediately
  (if (< nn 2) (static-exit agent 1)
      ;; otherwise, create a series of subtasks for computing the product of the first 
      ;; top values
      ;; we want to control the call for bid delay for the contract net to 
      ;; prevent early aborts
      ;; we initialize a :res value in the environment (used by dynamic function)
      (let ((*default-call-for-bids-timeout-delay* *timeout-delay*)
            (delay 0))
			(env-set agent 1 :res)
        ;; produce a limited number of initial subtasks, e.g., 3
        (dotimes (count 3)
          ;; create n/2 tasks for distribution to the multiplying agents
          ;; create a subtask for computing the product of the next two values
          ;; broadcast
          (send-subtask agent :to :ALL :action :multiply 
                        :args (list nn (decf nn)) :delay delay
                        :timeout *multiply-timeout* 
                        :protocol :contract-net)
          (decf nn) ; adjust nn for next round
          (incf *default-call-for-bids-timeout-delay* 3) ; increase waiting time
          ;;... to let agents make their bids
          (incf delay) ; we assume 1 as cost of setting up subtask
          ;; if nn becomes less than 3 don't multiply 2 by 1! get out of the loop
          (if (< nn 2)(return nil)))
        ;; define the :n tag in the environment to record the value of the next
        ;; products to compute. Carefull nn could be 0.
		(env-set agent (if (< nn 1) 1 nn) :n)
        ;; quit
        (static-exit agent :done))))

(defun dynamic-limited-fac (agent message answer)
  "this function is called whenever we get a result from a subtask. "
  (declare (ignore message))
  (let ((nn (env-get agent :n))
        (res (env-get agent :res))
        (list-of-subtasks (pending-subtasks? agent))
        ;(*default-call-for-bids-timeout-delay* *timeout-delay*)
        )
    ;(format t "~&Dynamic: nn=~S res=~s - (pending-subtasks? agent) ~S"
    ;        nn res list-of-subtasks)
    ;; we are finished when this was the last substasks and we do not have
    ;; partial result waiting (res=1)
    (cond 
     ((and (< nn 2) (null (cdr list-of-subtasks))(eql res 1))
      ;; thus we do a final exit, we just received the final answer
      (dynamic-exit agent answer))
     ;; if the recorded value is greater than 1, then we send a new subtask to th?
     ;; agent that sent the answer
     ((> nn 1)
      (send-subtask agent :to :ALL :action :multiply
                    :args (list answer nn) :timeout *multiply-timeout*
                    :protocol :contract-net)
      ;; update environment
	  (en-set agent (1- nn) :n)
      ;; then return an answer (actually to nobody in particular)
      answer)
     ;; otherwise we exhausted primary values, we must gather results of MUL
     ;; agents and multiply them together
     ((eql 1 res)
      ;; if 1 then we store partial result for next time around
	  (env-set agent answer :res)
      ;; then return an answer (actually to nobody in particular)
      answer)
     ;; otherwise we have a partial result in the environment
     ((not (eql 1 res))
      ;; use it to send a new multiply job
      (send-subtask agent :to :ALL :action :multiply
                    :args (list answer (cadr (member :res environment)))
                    :timeout *multiply-timeout*
                    :protocol :contract-net)
      ;; reset environment
	  (env-set agent 1 :res)
      ;; then return an answer (actually to nobody in particular)
      answer)
     ;; otherwise we are in trouble (something wrong with the algorithm)
     (t (error "bad limitied fac algorithm")))))

(defun timeout-limited-fac (agent message)
  "function for handling timeout errors"
  (case (omas::action message)
    (multiply
     ;; if a multiply agent is timed out, then reallocate to another one
     (send-subtask agent :to :ALL :action :multiply
                   :args (omas::args message)
                   :timeout *multiply-timeout*
                   :protocol :contract-net)
     ;; exit
     :done
     )
    (otherwise 
     ;; for any other skill we let the system process the timeout condition
     :unprocessed))
  )

; (trace (timeout-limited-fac :step t))
; (untrace timeout-limited-fac)

;;; ================================= environment ==============================

:EOF 