﻿;;;-*- Mode: Lisp; Package: "ORIBE" -*-
;;;===============================================================================
;;;11/04/04
;;;             AGENT ORIBE: ONTOLOGY (file ORIBE-ontology.lisp)
;;;             Copyright Barthès@UTC, 2009-2011
;;;
;;;=============================================================================== 

#|
ORIBE is the assistant agent of KS.
It can find email addressses, telephone numbers, or use web services (?)

Its ontology is used to process dialogs, answer requests or trigger actions, i.e.
send messages to other agents.
The current ontology contains information about addresses, publications and 
research specialties.
However, ORIBE is not a specialist of those subjects and its ontology is rather
shallow. It contains enough vocabulary to ask a specialist (one of the staff agent).

Questions asked to ORIBE:
   - what is the telephone number of "Trigano"?
   - phone "dbb"?
   - home phone of "Moulin"?
   - extension of DICIT?
   - send message to "gapenne".
   - ...
|#

(in-package :ORIBE)

(defontology 
  (:title "ORIBE ontology")
  (:version "1.0")
  (:language :en)
  )

(format t "~%;*** Loading ORIBE ontology, language is: ~S" *language*)

;;;===============================================================================
;;;                               ADDRESS
;;;===============================================================================

(defconcept "address"
  (:doc "An ADDRESS can be different things:
   - a postal home address
   - a professional address
   - an email
   - a web address, i.e. the address of a personal web page
Each of those can be multiple."))

;;;===============================================================================
;;;                             ADDRESS (EMAIL)
;;;===============================================================================

(defconcept "email address; email"
  (:is-a "address")
  (:doc "An EMAIL ADDRESS is the code allowing to receive an electronic message.")
  (:att "code"))

;;;===============================================================================
;;;                             ADDRESS (POSTAL)
;;;===============================================================================

(defconcept "postal address; home address"
  (:is-a "address")
  (:doc "A POSTAL ADDRESS od  HOME ADDRESS is the information needed to ~
         locate the place where a person is living.")
  (:att "street and number; street")
  (:att "town; city" (:unique))
  (:att "country" (:unique))
  (:att "zip; postal code" (:unique)))

;;;===============================================================================
;;;                             ADDRESS (WEB)
;;;===============================================================================

(defconcept "web address; web page; personal web page"
  (:doc "A WEB ADDRESS is the URL of a web page containing personal data.")
  (:att "URL"))
  
;;;===============================================================================
;;;                               COMPANY
;;;===============================================================================

(defconcept "company; firm; enterprise"
  (:is-a "organization")
  (:doc "A COMPANY, FIRM, or ENTERPRISE usually undertakes activities to make ~
         some profit for its owner(s).")
  )

;;;===============================================================================
;;;                             ORGANIZATION
;;;===============================================================================

(defconcept "organization"
  (:att "name" (:entry))
  (:att "acronym" (:entry))
  (:rel "address" (:to "address"))
  (:rel "phone" (:to "phone"))
  (:rel "web page" (:to "web page"))
  (:doc "An ORGANIZATION is a social structure private or public that ~
         has some social goals.")
  )

;;;===============================================================================
;;;                          ORGANIZATION (NON PROFIT)
;;;===============================================================================

(defconcept "non profit organization"
  (:is-a "organization")
  (:doc " A NON PROFIT ORGANIZATION is not allowed to derive profits from its ~
         activities.")
  )

;;;===============================================================================
;;;                               PERSON
;;;===============================================================================

(defconcept "person"
  (:att "name" (:entry))
  (:att "first-name" (:entry))
  (:rel "home address" (:to "home address"))
  (:rel "email" (:to "email"))
  (:rel "web page" (:to "web address"))
  (:rel "huspand" (:to "person")(:unique))
  (:rel "wife" (:to "person")(:unique))
  (:rel "mother" (:to "person"))
  (:rel "personal assistant" (:to "omas agent"))
  )

;;;===============================================================================
;;;                               STUDENT
;;;===============================================================================

(defconcept "student"
  (:is-a "person")
  (:rel "school" (:to "university"))
  )

;;;===============================================================================
;;;                             UNIVERSITY
;;;===============================================================================

(defconcept "university"
  (:is-a "organization")
  (:doc "A UNIVERSITY is an organization involved in higher education.")
  )

;;;===============================================================================
;;;                             TELEPHONE
;;;===============================================================================

(defconcept "phone; telephone"
  (:doc "A TELEPHONE is a means to talk when away. One can have different types ~
         of telephone number: home phone, office phone, cell phone, etc.")
  (:att "number")
  )
  
;;;===============================================================================
;;;                             TELEPHONE (CELL)
;;;===============================================================================

(defconcept "cell phone"
  (:doc "A CELL PHONE is a portable telephone.")
  (:is-a "phone")
  )
  
;;;===============================================================================
;;;                             TELEPHONE (HOME)
;;;===============================================================================

(defconcept "home phone; home telephone"
  (:doc "A HOME PHONE is a telephone installed at the private address of the owner.")
  (:is-a "phone")
  )
  
;;;===============================================================================
;;;                             TELEPHONE (OFFICE)
;;;===============================================================================

(defconcept "office phone; office telephone"
  (:doc "An OFFICE PHONE is a telephone installed at the work-place.")
  (:is-a "phone")
  )

  
;;;=============================================================================== 
;;;
;;;                                 METHODS
;;;
;;;=============================================================================== 

;;;-------------------------------------------------------------- (PHONE) =summary

(definstmethod =summary PHONE ()
  (list (format nil "~{~A~^, ~}" (HAS-NUMBER))))

;;;=============================================================================== 
;;;=============================================================================== 
;;;
;;;                                INDIVIDUALS
;;;
;;;=============================================================================== 
;;;=============================================================================== 

(defindividual "MOSS documentation"
  ("moss title" "global help")
  ("moss documentation" 
    "This is the global help... As your assistant I can do a certain ~
     number of things for you. They correspond to tasks I know of. The ~
	 easiest way to use me is to ask me using simple sentences what you ~
	 want to do.
     But remember, my vocabulary and understanding is quite limited.")
  )

;;;=============================================================================== 
  
;;; set of OMAS assistants

(defindividual "omas agent"
  ("ag-name" "albert")
  ("ag-key" :albert)
  (:language :fr)
  )
  
(defindividual "omas agent"
  ("ag-name" "alfredo")
  ("ag-key" :alfredo)
  (:language :es)
  )
  
(defindividual "omas agent"
  ("ag-name" "aristide")
  ("ag-key" :aristide)
  (:language :fr)
  )

(defindividual "omas agent"
  ("ag-name" "stevens")
  ("ag-key" :stevens)
  (:language :en)
  )
  
;;; Set of persons  
  
(defindividual "person"
  ("name" "Barthès")
  ("first name" "Jean-Paul")
  ("personal assistant" "albert")
  )
  
(defindividual "person"
  ("name" "Gonzales")
  ("first name" "Omar")
  ("personal assistant" ("omas agent" ("ag name" :is "alfredo")))
  )
  
(defindividual "person"
  ("name" "Ramos")
  ("first name" "Milton")
  ("personal assistant" ("omas agent" ("ag name" :is "aristide")))
  )
  
(defindividual "person"
  ("name" "Sugawara")
  ("first name" "Kenji")
  ("personal assistant" ("omas agent" ("ag name" :is "ORIBE")))
  )
;;;===============================================================================
:EOF