﻿;;;-*- Mode: Lisp; Package: "TYLER" -*-
;;;===============================================================================
;;;09/09/10
;;;             AGENT TYLER: DIALOG (file TYLER-dialog.lisp)
;;;             Copyright Barthes@UTC, 2009
;;;
;;;=============================================================================== 

#|
TYLER is the assistant agent of KS.
It can find email addressses, telephone numbers, or use web services (?)

The current file contains its library of dialogs.

Questions asked to TYLER:
   - what is the telephone number of Barthès?
   - phone Dominique?
   - home phone of Moulin?
   - extension of DICIT?
   - send message to claude.
   - ...
   
Available experimental and sketchy dialogs
   - MAIN DIALOG
   - ASK
   - EXPLAIN
   - GET ADDRESS
   - GET HOME ADDRESS
   - GET NEWS BY SECTION
   - GET PHONE NUMBER
   - GET LAST NEWS
   - PRINT CONCEPT DOCUMENTATION
   - PRINT HELP
   - PROCESS ASK
   - PROCESS TELL
   - PUBLISH NEWS
   - SET FONT SIZE
   - TELL
   - TRACE
   - WHAT IS
   - WHO IS
   - ELIZA
2007
 0530 WATCH OUT If we load this file with m-load, then the instances are being
      deferred for creation and _main-conversation for example is  not initialize
      until later. Thus all setq involving such variables will fail 
2009
 0603 upgrading file to version 7.9
 07720 correcting a bug in =answer-analysis _pa-get-answer
|#

(in-package :TYLER)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (setq *language* :en)
  )

(format t "~%;*** Loading TYLER dialogs, package is: ~S" *package*)

(defparameter *empty-words* '("a" "an" "the"))

;;;================================================================================

;;; here the strategy is to develop a specific dialog for each task. However, we 
;;; could also use a generic dialog consisiting in:
;;;    - getting the list of required parameters
;;;    - for each parameter, trying to extract info from data using predefined
;;;      rules or extraction patterns (Note that this is not much different than
;;;      having a predefined dialog, albeit focused on a single parameter)
;;;    - if some required parameters are missing, ask master about them using 
;;;      predefined questions
;;;    - shipping result to ad hoc staff agent

;;; The generic dialog could be an option offered to the programmer, which requires
;;; a suitable modeling of the tasks.


;;;================================================================================
;;;
;;;                              Sub-Conversations
;;;
;;;================================================================================

;;; Sub-conversations are defined by a conversation header, that points to the
;;; entry state.
;;; There are two output states:
;;;   SUCCESS: the result is stored into the RESULTS slot of the state-context
;;;   FAILURE: no result could be achieved
;;; In case of bad error there is a throw to an :error label
;;; data are obtained from the HAS-DATA slot of the context object

;;;================================================================================
;;;
;;;                      GENERIC TASK CONVERSATION
;;;
;;;================================================================================

;;; this subconversation takes a task model and fills in the required parameters
;;; before shipping the data (for possible optional args) and filled pattern to the 
;;; specialist

;;; probably not very useful

;;;================================================================================
;;;
;;;                           ASK CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to ask a remote PA about something. However, the
;;; ASK conversation is not very useful. It can be replaced by the TELL conversation
;;; The rationale is that the receiver will not answer right away, thus if we send
;;; a request, the corresponding task will die after 1 hour. Thus ASK sends in
;;; practice an INFORM TELL message.

;;; We must first locate our receiver (presumably the PA of somebody in the coterie
;;; We then want to prepare topic and text, then send the message

(eval-when (compile load eval)
  (proclaim '(special 
              _ask-entry-state  ; required by defstate            
              _ask-choose-destination 
              _ask-get-data
              _ask-get-destination
              _ask-send-message
              _ask-sorry
              )))

;;;================================================================================
;;;                       ASK CONVERSATION SERVICE FUNCTIONS
;;;================================================================================

;;;------------------------------------------------------------ EXTRACT-DESTINATION

(defun extract-destination (text-list)
  "tries to obtain the key of a PA as a destination for a message.
Arguments:
   text-list: list of words
Return:
   a PA keyword a list of persons or nil in case of failure."
  (let (entry-point-list key pa)
    (cond
     ;; if text-list is nil, then error
     ((null text-list)
      (return-from extract-destination nil))
     ;; try to locate the name of an agent
     ((setq entry-point-list 
            (moss::find-best-entries-from-property-name-and-class-ref
             text-list 'HAS-NAME "omas-agent" :keep-results t))
      ;; agent have unique names, thus take the first element from the list
      (setq key (car (has-key (cadar entry-point-list))))
      (return-from extract-destination key))
     ;; could not find agent, try to locate a person from name or first name
     ((setq entry-point-list 
            (moss::find-best-entries-from-property-name-and-class-ref
             text-list 'HAS-name "person" :keep-results t))
      ;; if a list return the list of person IDs
      (when (cdr entry-point-list)
        (return-from extract-destination (mapcar #'cadr entry-point-list))))
     ;; could not fing person from names, try first names
     ((setq entry-point-list 
            (moss::find-best-entries-from-property-name-and-class-ref
             text-list 'HAS-first-name "person" :keep-results t))
      (when (cdr entry-point-list)
        (return-from extract-destination (mapcar #'cadr entry-point-list))))
     ;; could not find anything
     (t (return-from extract-destination nil)))
    ;(print entry-point-list)
    ;; here we have a single answer
    (setq pa (car (send (cadar entry-point-list) '=get "personal assistant")))
    ;; and agent key
    (car (has-key pa))))

#|
(extract-destination nil)
NIL
(extract-destination '("albert"))
:ALBERT
(extract-destination '("ramos"))
:ALBERT
(extract-destination '("milton"))
:ALBERT
(extract-destination '("envoyer" "un" "message" "à" "milton"))
:ALBERT
(extract-destination '("milton" "ramos"))
:ALBERT
|# 
;;;================================================================================
;;;                            ASK CONVERSATION STATES
;;;================================================================================

;;;--------------------------------------------------------- (ASK) ASK-CONVERSATION

(defsubdialog 
  _ask-conversation 
  (:label "Ask conversation")
  (:explanation 
   "We want our PA to ask something to somebody.")
  )

;;;---------------------------------------------------------- (ASK) ASK-ENTRY-STATE

(defstate 
  _ask-entry-state
  (:label "Ask dialog entry")
  (:entry-state _ask-conversation)
  (:explanation "We try to get destination from the input data.")
  (:answer-analysis)
  )


(defownmethod
  =answer-analysis _ask-entry-state (conversation input)
  "Using list of words to locate receiver of message to send. Must ask user ~
   to confirm findings.
Arguments:
   conversation: conversation object
   input copy of the FACTS/input area"
  (let* (bindings destination)
    (catch 
      :return
      (cond
       ;; when input is empty then failure
       ((null input)
        (throw :return (list :failure)))
       ;; try to extract destination info from the input text
       ((setq bindings
              (MOSS::PROCESS-PATTERNS 
               input
               '(((?* ?X) "ask" (?* ?Y)) 
                 ((?* ?X) "tell" (?* ?Y)) 
		         ((?* ?X) "message" "for" (?* ?Y))
                 ((?* ?X) "message" "to" (?* ?Y))
                 )))
        (setq input (cdr (assoc '?Y bindings)))
        )
       ;; otherwise do nothing
       )
      
      ;; try to get destination
      (setq destination (extract-destination input))
      
      (cond
       ;; if null, failure
       ((null destination)
        (throw :return (list :transition _ask-sorry)))
       ;; list, must make a choice
       ((listp destination)
        ;; save the result into the FACT area and go ask for choice
        (replace-fact conversation :to-choose destination)
        (throw :return (list :transition _ask-choose-destination)))
       ;; otherwise we got destination, save it
       (t
        (replace-fact conversation :to destination)
        ;; and go get data
        (throw :return (list :transition _ask-get-data))))
      )))

;;;------------------------------------------------------- (ASK) CHOOSE-DESTINATION

(defstate 
  _ask-choose-destination
  (:label "Ask choose destination")
  (:explanation "ambiguous destination (list) select one.")
  (:execute
   (let* ((conversation moss::conversation)
          (choices (read-fact conversation :to-choose))
          )
     ;; remove the temporary data
     (replace-fact conversation :to-choose nil)
     (replace-fact conversation :to (car choices))))
  (:transitions (:always :target _ask-get-data)))

;;;----------------------------------------------------------------- (ASK) GET-DATA

(defstate 
  _ask-get-data
  (:label "set up parameters for continuous input.")
  (:explanation "Let the master type text into the pane until the DONE button is ~
                 pushed. We must disable . and ? terminations.")
  ;; set global flag in the interface window so that we can have . and ? in the text
  (:execute-preconditions 
   (setf (omas::pass-every-char (car (has-input-window moss::conversation))) t)
   ;; include :verbatim flag into the FACT base (?)
   (replace-fact moss::conversation :verbatim t)
   )
  (:question-no-erase 
   "~% ...Please type or dictate your text then puis click the End button.")
  (:transitions
   (:always :target _ask-send-message))
  )

;;;---------------------------------------------------------- (ASK) GET-DESTINATION

(defstate 
  _ask-get-destination
  (:label "obtain master's destination for message.")
  (:explanation "Get the id of the personal assistant of the receiver.")
  ;; set global so that we can have . and ? in the text
  (:question-no-erase 
   "~% To whom do you want to send your message?") 
  (:answer-analysis)
  )

(defownmethod =answer-analysis _ask-get-destination (conversation input)
  "We process the answer and record it unless we get an abort.
Arguments:
   agent: assistant."
  (let (destination)
    ;; analyze input
    (setq destination (extract-destination input))
    
    (cond
     ((null destination)
      (list :transition _ask-sorry))
     ((listp destination)
      (replace-fact conversation :to-choose destination)
      (list :transition _ask-choose-destination))
     (t
      (replace-fact conversation :to destination)
      ;; we then get out of dialog
      (list :transition _ask-get-data)))))

;;;------------------------------------------------------------- (ASK) SEND-MESSAGE

(defstate 
  _ask-send-message
  (:label "Ask send final message")
  (:explanation "build and send the message.")
  (:send-message-no-wait 
   :to (pop-fact moss::conversation :to)
   :type :inform
   :action :tell
   :args (list (list :text (pop-fact moss::conversation :raw-input))))
  (:text "~&...Message was sent.")
  (:transitions (:always :success)) ; return
  )

;;;-------------------------------------------------------------------- (ASK) SORRY

(defstate 
  _ask-sorry
  (:label "Ask failure.")
  (:explanation "We don't have the needed parameters")
  (:text 
   "Sorry, I cannot figure out to whom you want to send the message.~%")
  (:reset)
  (:transitions (:failure))
  )

;;;================================================================================
;;;
;;;                          EXPLAIN CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to explain the meaning of a concept.

(eval-when (compile load eval)
  (proclaim '(special 
              _explain-conversation ; unnecessary defsubdialog does a declaim
              _exp-entry-state  ; required by defstate
              )))

;;;---------------------------------------------------- (EXP) EXPLAIN-CONVERSATION

(defsubdialog 
  _explain-conversation 
  (:label "Explain conversation")
  (:explanation 
   "The data is searched for a concept to be explained.")
  )

;;;----------------------------------------------------- (EXP) EXPLAIN-ENTRY-STATE

(defstate 
  _exp-entry-state
  (:label "Explain dialog entry")
  (:entry-state _explain-conversation)
  (:explanation "The data is cleaned prior to look for an entry point.")
  (:transitions
   ;;remove useless words explain, define, or else...
   (:patterns (((?* ?x) "define" (?* ?y))
               ((?* ?x) "definition" (?* ?y))
               ((?* ?x) "explain" (?* ?y))
               ((?* ?x) "what" "means" (?* ?y))
               ("what" "is" (?* ?y) "?")
               ((?* ?y) "is" "what" (?* ?x))
               )
              :keep ?y ; put a list of the binding value into context HAS-DATA
              :sub-dialog _print-concept-documentation-conversation)
   ;; we do not know how the user got here and keep everything
   (:otherwise :sub-dialog _print-concept-documentation-conversation)
   )
  )

;;;================================================================================
;;;
;;;                      GET ADDRESS CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to print the address of a person or of a
;;; place. It fills the task pattern and sends the message to the address specialist.
;;; The simplest version is to send to the :SA-ADDRESS agent, wait for the result and
;;; print it.

(eval-when (compile load eval)
  (proclaim '(special 
              _get-address-dialog  ; required by defstate
              _gad-entry-state
              _gad-dont-understand
              _gad-try-again
              _gad-sorry)))

;;;------------------------------------------------ (GAD) GET-ADDRESS-CONVERSATION

(defsubdialog 
  _get-address-conversation 
  (:label "Get address conversation")
  (:explanation 
   "Master is trying to obtain an address.")
  )

;;;================================================================================
;;;                  GET ADDRESS CONVERSATION SERVICE FUNCTIONS
;;;================================================================================

;;;============================ printing adresses ================================
;;;********** this should be improved to handle office address and other particular
;;; types of addresses

;;;------------------------------------------------------------------- GET-ADDRESS

(defun get-address (pattern &key (kind "adresse postale"))
  "try to extract from a structured description the sub-structure describing the ~
   address of the required type.
    (\"personne\" (\"nom\" \"Barthés\") (\"prénom\" \"Jean-Paul\") 
       (\"adresse\" 
            (\"domicile\"
                  (\"rue\" \"14, Allée de la Tilloye\") (\"code postal\" \"60200\")
                  (\"ville\" \"Compiégne\")
                  (\"pays\" \"France\")))
Arguments:
   pattern: the structured data
   kind (key): the type of address required
Return:
   the corresponding sub-structure or nil."
  ;; if the pattern does not contain the "address" property, then return nil
  (let ((address (assoc "address" (cdr pattern) :test #'string-equal)))
    (print address)
    (when address
      ;; otherwise, check for the corresponding types of addresses
      (remove nil
              (mapcar #'(lambda (xx) (print (car xx)) 
                         (if (moss::%pclass? (car xx) kind) xx))
                      (cdr address))))))
#|
(setq *language* :fr)
(setq pattern '("personne" ("nom" "Barthes") ("prénom" "Jean-Paul")
                ("adresse"
                 ("domicile" ("rue" "14, Allée de la Tilloye")
                  ("ville" "Compiègne") ("pays" "France")
                  ("code postal" "60200"))
                 ("adresse professionnelle"
                  ("rue" "Université de Technologie de Compiègne"
                   "Centre de Recherche de Royallieu"
                   "Departement de Génie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compiègne cedex") ("code postal" "60206")))
                ("téléphone"
                 ("téléphone du domicile"
                  ("numéro" "+33 (0)3 44 23 31 37"))
                 ("téléphone professionnel"
                  ("numéro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("numéro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))

(get-address albert::pattern)
(get-address albert::pattern :kind "domicile")
(get-address albert::pattern :kind "adresse professionnelle")
|#

(defun str-equal (aa bb)
  "make sure arguments are strings before comparing."
  (and (stringp aa)(stringp bb)(string-equal aa bb)))

;;;----------------------------------------------------------------------- PCLASS?

(defun pclass? (input class-ref)
  "tests if input belongs to the transitive closure of class-ref. Only works in ~
   the ALBERT package. The language is important.
Arguments:
   input: some expression that must be a string
   class-ref: a string naming a class
Return:
   nil if not the case, something not nil otherwise."
  (member input (moss::%get-subclass-names class-ref) :test #'string=))

#|
? (pclass? (car '("étudiant"
                  ("nom" "Sato") ("prénom" "Gilson")
                  ("adresse"
                   ("adresse postale" 
                    ("rue" "Centre de Recherche de Royallieu" "BP 20529")
                    ("ville" "Compiégne cedex") ("code postal" "60205")))))
           "personne")
("étudiant")
|#
;;;----------------------------------------------------------------- PRINT-ADDRESS

(defun print-address (addr-list)
  ;(format t "~&+++ ALBERT/ print-address language: ~S address:~% ~S" 
  ;         *language* addr-list)
  (omas::assistant-display-text 
   PA_TYLER
   (format nil "~{~A~^~%~}" 
           (mapcar #'print-single-address addr-list))))

;;;------------------------------------------------------------ PRINT-HOME-ADDRESS

(defun print-home-address (addr-list)
  ;(format t "~&+++ TYLER/ print-address language: ~S address:~% ~S" 
  ;         *language* addr-list)
  (omas::assistant-display-text 
   PA_TYLER
   (format nil "~{~A~^~%~}" 
           (mapcar #'print-single-home-address addr-list))))

;;;---------------------------------------------------------- PRINT-SINGLE-ADDRESS

(defun print-single-address (addr)
  "input is something like:
     (\"personne\" (\"nom\" \"Barthes\") (\"prénom\" \"Jean-Paul\") 
       (\"adresse\" 
            (\"adresse du domicile\"
                  (\"rue\" \"14, Allée de la Tilloye\") (\"code postal\" \"60200\")
                  (\"ville\" \"Compiégne\")
                  (\"pays\" (\"pays\" (\"nom\" \"France\")))))
   to obtain
     Barthes, Jean-Paul
        adresse du domicile:
           14, allée de la ...
           60200 Compiégne
           France
   or something similar for companies."
  (let (result details)
    (cond
     ;; first test person or subclass
     ((pclass? (car addr) "person")
      (push (moss::%pformat "~{~A~^ ~}, ~{~A~^ ~}" (cdr addr)
                            "name" "first name") 
            result))
     ;; then test organization or subclass
     ((moss::%pclass? (car addr) "organization")
      (push (moss::%pformat  "~{~A~^, ~}" (cdr addr) "name") result)))
    
    ;; then process the address itself (if there)
    (cond
     ;; address? (check against all synonyms and subclasses)
     ((setq details (moss::%get-value-from-class-ref "address" (cdr addr)))
      ;; get standard address  (("street" "14, ...")(...)...)
      ;; for each type of address print the corresponding info
      (dolist (item details)
        (push (format nil "   ~A:" (car item)) result)
        (push (moss::%pformat  
               "~5T~{~A~^~&~5T ~}~%~5T~{~A~^ ~} ~{~A~^ ~}"
               (cdr item)
               "street" "postal code" "city")
              result)
        ;(format *debug-io* "~% item: ~S" item)
        ;; country is different since it is a structured object
        (let* ((expr (car (moss::%get-value-from-prop-ref "country" (cdr item))))
               (country-name (moss::%get-value-from-prop-ref "name" (cdr expr))))
          ;(format *debug-io* "~% expr: ~S; country-name: ~S" expr country-name)
          (if country-name (push (format nil "~5T~{~A~^ ~}" country-name)
                                 result)))
        )))
    
    ;; now print
    (format nil "~{~A~^~%~}" (reverse result))))

#|
(setq pattern '("personne" ("nom" "Barthes") ("prénom" "Jean-Paul")
                ("adresse"
                 ("domicile" ("rue" "14, Allée de la Tilloye")
                  ("ville" "Compiègne") ("pays" "France")
                  ("code postal" "60200"))
                 ("adresse professionnelle"
                  ("rue" "Université de Technologie de Compiègne"
                   "Centre de Recherche de Royallieu"
                   "Departement de Génie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compiègne cedex")
                  ("pays" ("pays" ("nom" "France")))
                  ("code postal" "60206")))
                ("téléphone"
                 ("téléphone du domicile"
                  ("numéro" "+33 (0)3 44 23 31 37"))
                 ("téléphone professionnel"
                  ("numéro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("numéro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))

? (moss::%get-value-from-class-ref "adresse" (cdr pattern))
(("domicile" ("rue" "14, Allée de la Tilloye") ("ville" "Compiègne") ("pays" "France")
  ("code postal" "60200"))
 ("adresse professionnelle"
  ("rue" "Université de Technologie de Compiègne" "Centre de Recherche de Royallieu"
   "Departement de Génie Informatique" "UMR CNRS 6599 HEUDIASYC")
  ("ville" "Compiègne cedex") ("code postal" "60206")))

? (print-single-address '("personne" ("nom" "Barthes") ("prénom" "Jean-Paul")
                          ("adresse"
                           ("domicile" ("rue" "14, Allée de la Tilloye")
                            ("ville" "Compiègne") ("pays" ("pays" ("nom" "France")))
                            ("code postal" "60200"))
                           ("adresse professionnelle"
                            ("rue" "Université de Technologie de Compiègne"
                             "Centre de Recherche de Royallieu"
                             "Departement de Génie Informatique"
                             "UMR CNRS 6599 HEUDIASYC")
                            ("ville" "Compiègne cedex")
                            ("pays" ("pays" ("nom" "France")))
                            ("code postal" "60206")))))
"Barthes, Jean-Paul
   domicile:
     14, Allée de la Tilloye
     60200 Compiègne
     France
   adresse professionnelle:
     Université de Technologie de Compiègne
      Centre de Recherche de Royallieu
      Departement de Génie Informatique
      UMR CNRS 6599 HEUDIASYC
     60206 Compiègne cedex
     France"
|#
;;;----------------------------------------------------- PRINT-SINGLE-HOME-ADDRESS

(defun print-single-home-address (addr)
  (let (result details)
    (cond
     ;; first test person or subclass
     ((pclass? (car addr) "person")
      (push (moss::%pformat "~{~A~^ ~}, ~{~A~^ ~}" (cdr addr)
                            "name" "first name") 
            result)))
    
    ;; then process the address itself (if there)
    ;; address? (check against all synonyms and subclasses)
    (setq details (get-address addr :kind "home address"))
    ;; get standard address  (("street" "14, ...")(...)...)
    ;; for each type of address print the corresponding info
    (cond 
     (details
      (dolist (item details)
        (push (format nil "   ~A:" (car item)) result)
        (push (moss::%pformat  
               "~5T~{~A~^~&~5T ~}~%~5T~{~A~^ ~} ~{~A~^ ~}~%~5T~{~A~^ ~}"
               (cdr item)
               "street and number" "zip" "town" "country")
              result)))
     (t (push "   I do not know the home address...
" result)))    
    ;; now print
    (format nil "~{~A~^~%~}" (reverse result))))

#|
(setq pattern '("personne" ("nom" "Barthes") ("prénom" "Jean-Paul")
                ("adresse"
                 ("domicile" ("rue" "14, Allée de la Tilloye")
                  ("ville" "Compiègne") ("pays" "France")
                  ("code postal" "60200"))
                 ("adresse professionnelle"
                  ("rue" "Université de Technologie de Compiègne"
                   "Centre de Recherche de Royallieu"
                   "Departement de Génie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compiègne cedex") ("code postal" "60206")))
                ("téléphone"
                 ("téléphone du domicile"
                  ("numéro" "+33 (0)3 44 23 31 37"))
                 ("téléphone professionnel"
                  ("numéro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("numéro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))
(print-single-home-address pattern)
(setq pattern '("personne" ("nom" "Barthes") ("prénom" "Jean-Paul")
                ("adresse"
                 
                 ("adresse professionnelle"
                  ("rue" "Université de Technologie de Compiègne"
                   "Centre de Recherche de Royallieu"
                   "Departement de Génie Informatique"
                   "UMR CNRS 6599 HEUDIASYC")
                  ("ville" "Compiègne cedex") ("code postal" "60206")))
                ("téléphone"
                 ("téléphone du domicile"
                  ("numéro" "+33 (0)3 44 23 31 37"))
                 ("téléphone professionnel"
                  ("numéro" "+33 (0)3 44 23 44 66"))
                 ("portable" ("numéro" "+33 (0)6 80 45 32 67")))
                ("email" ("email" ("e-adresse" "barthes@utc.fr")))
                ("page web"
                 ("adresse web" ("URL" "http://www.utc.fr/~barthes")))))
|#
;;;================================================================================
;;;                        GET ADDRESS CONVERSATION STATES
;;;================================================================================

;;;----------------------------------------------------- (GAD) ADDRESS-ENTRY-STATE

(defstate 
  _get-address-entry-state
  (:entry-state _get-address-conversation)
  (:label "Get address dialog entry")
  (:explanation "Assistant is sending a free-style message to the ADDRESS agent.")
  (:send-message :to :CIT-ADDRESS :self PA_TYLER :action :get-address
                 :args `(((:data . ,(moss::read-fact moss::conversation :input))
                          (:language . :en)))
                 )
  (:transitions
   (:on-failure :target _gad-dont-understand)
   (:test (> (length (read-fact moss::conversation :answer)) 3)
          :exec
          (progn
            ;; record the pattern for printing selection summaries into the working 
            ;; list
            (replace-fact moss::conversation :control-string "~{~A~^ ~}, ~{~A~^ ~}")
            (replace-fact moss::conversation :properties (list "name" "first name"))
            )
          :sub-dialog _make-choice-conversation :failure _gad-dont-understand)
   (:otherwise
    :print-answer #'print-address :success))
  )

;;;----------------------------------------------------- (GAD) GAD-DONT-UNDERSTAND

(defstate 
  _gad-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "could not find the address with the given information. Thus, ~
                 asking master for whose address...")
  (:question-no-erase "~%- Whose address?")
  (:answer-type :answer)
  (:transitions
   (:always :target _gad-try-again)))

;;;--------------------------------------------------------------- (GAD) GAD-SORRY

(defstate 
  _gad-sorry
  (:label "Address failure.")
  (:explanation "We don't have the requested address.")
  (:text "Sorry, if you were looking for an address, I can't find it.~%")
  (:reset)
  (:transitions (:failure))
 )

;;;----------------------------------------------------------- (GAD) GAD-TRY-AGAIN

(defstate 
  _gad-try-again
  (:label "Ask again for address.")
  (:explanation "Assistant is asking :CIT-ADDRESS.")
  (:send-message :to :CIT-ADDRESS :self PA_TYLER :action :get-address
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en))))
  (:transitions
   (:on-failure :target _gad-sorry)
   (:otherwise
    :print-answer #'print-address
    :success))
  )

;;;================================================================================
;;;
;;;                      GET HOME ADDRESS CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to print the home address of a person.
;;; It builds a task pattern and sends the message to the address specialist.
;;; The simplest version is to send to the :CIT-ADDRESS agent, wait for the result and
;;; print it.

(eval-when (compile load eval)
  (proclaim '(special 
              _get-home-address-conversation  ; required by defstate
              _ghad-entry-state
              _ghad-dont-understand
              _ghad-process-answer
              _ghad-try-again
              _ghad-try-again-process-answer
              _ghad-sorry)))

;;;------------------------------------------ (GHAD) GET-HOME-ADDRESS-CONVERSATION

(defsubdialog 
  _get-home-address-conversation 
  (:label "Get address conversation")
  (:explanation 
   "Master is trying to obtain a private address.")
  )

;;;------------------------------------------- (GHAD) GET-HOME-ADDRESS-ENTRY-STATE

(defstate 
  _get-home-address-entry-state
  (:entry-state _get-home-address-conversation)
  (:label "Get home address dialog entry")
  (:explanation "Assistant is sending a free-style message to the ADDRESS agent.")
  (:send-message :to :CIT-ADDRESS :self PA_TYLER :action :get-address
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          #|
(:pattern . ("person" 
                                       ("name")
                                       ("first name")
                                       ("address" 
                                        ("home address" 
                                         ("street") ("postal code") ("city")))))
|#)))
  (:answer-analysis)
  )

(defownmethod =answer-analysis _get-home-address-entry-state (conversation input)
  "Home address: we got an answer from the ADDRESS message or else the master got ~
   impatient.
Arguments:
   agent: assistant."
  (declare (ignore input))
  (let* ((agent (car (has-agent conversation)))
         (answer (omas::answer agent))
         (*language* :en)
         )
    ;; answer contains either the word "failure" or a list of objects expressed as
    ;; strings
    ;; (("personne" ("nom" "Barthes") ("prénom" "Jean-Paul") :home-phone "...")*)
    ;; 
    (moss::vformat "~&Answer to the send-request: ~S" answer)
    ;; on failure do something
    (cond
     ((equal "failure" answer)
      `(:transition ,_ghad-dont-understand))
     ;; if more than 3 persons, then ask the user to make a choice
     ((> (length answer) 3)
      ;; record answer into results
      (replace-fact conversation :results answer)
      ;; record the pattern for printing selection summaries into the working list
      (replace-fact conversation :control-string "~{~A~^ ~}, ~{~A~^ ~}")
      (replace-fact conversation :properties (list "name" "first name"))
      ;; then call sub-conversation
      `(:sub-dialog _make-choice-conversation
                    ;:success :success  ; normally the default
                    :failure ,_ghad-dont-understand))
     (t 
      ;(format t "~&+++ address: ~S" input)
      (print-home-address answer)
      `(:success)))))

;;;-------------------------------------------------------- (GHAD) DONT-UNDERSTAND

(defstate 
  _ghad-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "could not find the address with the given information. Thus, ~
                 asking master for whose address...")
  (:question "~%- Whose home address?")
  (:answer-type :answer)
  (:transitions
   (:always :save :target _ghad-try-again)))

;;;------------------------------------------------------------------ (GHAD) SORRY

(defstate 
  _ghad-sorry
  (:label "Address failure.")
  (:explanation "We don't have the requested address.")
  (:text 
   "Sorry, if you were looking for a home address, I can't find it.~%")
  (:reset)
  (:immediate-transitions (:failure))
  (:no-resume))

;;;-------------------------------------------------------------- (GHAD) TRY-AGAIN

(defstate 
  _ghad-try-again
  (:label "Ask again for address.")
  (:explanation "Assistant is asking :CIT-ADDRESS.")
  (:send-message :to :CIT-ADDRESS :self PA_TYLER :action :get-address
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          (:pattern . ("person" 
                                       ("name")
                                       ("first name")
                                       ("address" 
                                        ("home address" 
                                         ("street") ("postal code") ("city"))))))))
  (:transitions
   (:on-failure :target _ghad-sorry)
   (:otherwise
    :print-answer #'print-home-address)
   )
  )

  

;;;================================================================================
;;;
;;;                      GET NEWS BY SECTION CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to print news about a specific topic. It fills 
;;; the task pattern and sends the message to the news specialist.

(eval-when (compile load eval)
  (proclaim '(special 
              _get-news-by-section-dialog  ; required by defstate
              _gnbsd-entry-state
              _gnbsd-dont-understand
              _gnbsd-try-again
              _gnbsd-sorry)))

;;;================================= printing news ================================

;;;--------------------------------------- (GNBSD) GET-NEWS-BY-SECTION-CONVERSATION

(defsubdialog 
  _get-news-by-section-conversation 
  (:label "Get news by section conversation")
  (:explanation 
   "Master is trying to obtain news of a specific section.")
  )

(defstate 
  _get-news-by-section-entry-state
  (:entry-state _get-news-by-section-conversation)
  (:label "Get news by section dialog entry")
  (:explanation "Assistant is sending a free-style message to the NEWS agent.")
  (:send-message :to :NEWS :self PA_TYLER :action :get-news-by-section
                 :args `(((:data . ,(moss::read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 )
  (:transitions
     (:on-failure :target _gnbsd-dont-understand)
	 (:otherwise :exec (print-news (read-fact PA_TYLER :answer)) :success))
  )

;;;--------------------------------------------------- (GNBSD) GNBS-DONT-UNDERSTAND

(defstate 
  _gnbsd-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "could not find news in the requested section. Thus, ~
                 asking master for what section...")
  (:question-no-erase "~%- What topic (section) do you mean exactly?")
  (:answer-type :answer)
  (:transitions
   (:always  :target _gnbsd-try-again)))

;;;------------------------------------------------------------------ (GNBSD) SORRY

(defstate 
  _gnbsd-sorry
  (:label "News obtention failure.")
  (:explanation "We don't have the requested news.")
  (:text "Sorry, I could not find anything.~%")  
  (:reset)
  (:immediate-transitions (:failure))
  (:no-resume))

;;;-------------------------------------------------------------- (GNBSD) TRY-AGAIN

(defstate 
  _gnbsd-try-again
  (:label "Ask again for news.")
  (:explanation "Assistant is asking :NEWS.")
  (:send-message :to :NEWS :self PA_TYLER :action :get-news-by-section
                 :args `(((:data . ,(moss::read-fact moss::conversation :input))
                          (:language . :en))))
    (:transitions
     (:on-failure :target _gnbsd-dont-understand)
	 (:otherwise :exec (print-news (read-fact PA_TYLER :answer)) :success))
	)
 
;;;===============================================================================
;;;
;;;                       GET LAST NEWS CONVERSATION
;;;
;;;===============================================================================

;;; this conversation is intended to show the 5 last published news

(eval-when (compile load eval)
  (proclaim '(special 
              _GET-LAST-NEWS-CONVERSATION  ; required by defstate
              _gln-entry-state
              _gln-dont-understand
              _gln-try-again
              _gln-sorry)))


;;;-------------------------------------------------------------- PRINT-SINGLE-NEW

(defun print-news-item (new)
   "prints the title (a list of words) and the content (a list containing a string)."
  (format nil "~A ~A (~A) : ~{~A ~}~%    ~A" 
    (cadr (assoc :current-date new))
	(cadr (assoc :current-hour new))
	(or (cadr (assoc :author new)) "anonyme")
	(or (cdr (assoc :title new)) (list "no title"))
    (cadr (assoc :content new))))
	
;;;-------------------------------------------------------------------- PRINT-NEWS

(defun print-news (news-list)
  ;(format t "PARAMETRO A PRINT-NEWS ~S~%" news-list)
  (omas::assistant-display-text 
  PA_TYLER
  (format nil "~{~A~^~%~}" (mapcar #'print-news-item news-list))))

(defsubdialog
  _GET-LAST-NEWS-CONVERSATION
  (:label "Get the more recent news")
  (:explanation 
   "This dialog is meant to show the last news.")
  )

;;;---------------------------------------------------- (gln) GET-LAST-NEWS-DIALOG
;;;********** just check if :answer contains the answer and the "failure" mechanism

(defstate 
  _gln-entry-state
  (:label "Get last news dialog")
  (:entry-state _get-last-news-conversation)
  (:explanation 
   "Sends message to the news agent, no arguments are necessary.")
  (:send-message :to :NEWS :self PA_TYLER :action :get-last-news
                 :args `(((:language . :en))))
  (:transitions
    (:on-failure :target _gln-sorry)
	(:otherwise :exec (print-news (read-fact moss::conversation :answer))
            	:success))
  )

#|
(defownmethod =resume _gln-entry-state (conversation &rest more-args)
  "we got an answer from the NEWS Agent or else the master got impatient.
Arguments:
   agent: assistant."
  (declare (ignore more-args))
  
  (let* ((agent (car (has-agent conversation)))
         (answer (omas::answer agent))
         (*language* :fr)
         )        
    ;; clean up HAS-TO-DO (contains "answer-there")
    ;(setf (has-to-do conversation) nil)
     
    (moss::vformat "_gln-entry-state /Answer to the send-request:~& ~S" answer)
    
    ;; on failure move to another state
    (cond     
     ((equal "failure" answer)
      `(:transition ,_gln-sorry))
     ;;;Otherwise print obtained news
     (t 
      (format t "CONDICION T")
      (print-news answer)
      `(:success)))))
|#

;;;------------------------------------------------------------------- (GLN) SORRY

(defstate 
  _gln-sorry
  (:label "Obtaining news failure.")
  (:explanation "NEWS failed to obtain news.")
  (:answer "~%- ~%- I could not find anything. ~
            Maybe there are no news on this topic ?")
  (:reset)			
  (:transitions 
   (:always :failure))
  )
  
;;;================================================================================
;;;
;;;                      GET TELEPHONE NUMBER CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to print the telephone number of a person or of a
;;; place. It fills the task pattern and sends the message to the address specialist.
;;; The simplest version is to send to the :SA-ADDRESS agent, wait for the result and
;;; print it.

;; create a T-CONTEXT from the task description. The T-CONTEXT contains the dynamic
;; information corresponding to the particular instance of this conversation.
;; Could we put this info into the Q-CONTEXT with additional properties?

;; fill it with the arguments

;; put the rest of the data into the reouired data argument

;; ship it to the specialist

;; wait to see what happens

(eval-when (compile load eval)
  (proclaim '(special 
              _get-phone-conversation  ; required by defstate
              _gpd-entry-state
              _gpd-dont-understand
              _gpd-try-again
              _gpd-sorry)))

;;;-------------------------------------------------------------------- MAKE-ALIST

(defun make-alist (ll)
  (cond ((null ll) nil)
        (t (cons (list (car ll)(cadr ll)) 
                 (make-alist (cddr ll))))))
#|
? (make-alist '(:OWNER ($E-PERSON.1) :CELL-PHONE "(0)6 80 45 32 67" :OFFICE-PHONE
  "(0)3 44 23 44 66" :HOME-PHONE "(0)3 44 23 31 37"))
((:OWNER ($E-PERSON.1)) (:CELL-PHONE "(0)6 80 45 32 67")
 (:OFFICE-PHONE "(0)3 44 23 44 66") (:HOME-PHONE "(0)3 44 23 31 37"))
|#
;;;-------------------------------------------------------- (GPD) GET-PHONE-DIALOG

;;; The simplest use is to take text in the HAS-DATA slot of the Q-CONTEXT
;;; and send a message to :SA-ADDRESS

(defsubdialog 
  _get-phone-conversation
  (:label "Get phone dialog")
  (:explanation "Master is trying to obtain a telephone number.")
  )

;;;------------------------------------------------------------- (GPD) ENTRY-STATE

(defstate 
  _gpd-entry-state
  (:label "Get phone dialog")
  (:entry-state _get-phone-conversation)
  (:explanation "Assistant is sending a free-style message to the ADDRESS agent.")
  (:send-message :to :CIT-ADDRESS :self PA_TYLER :action :get-phone-number
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          #|
(:pattern . ("person" ("name")("first name")
             ("phone" ("telephone"))))
|#)))
  (:transitions
   (:on-failure :target _gpd-dont-understand)
   (:otherwise
    :exec (omas::assistant-display-text 
           PA_TYLER (moss::make-print-list (read-fact moss::conversation :answer)))
    :success))
  )

;;;----------------------------------------------------- (GPD) GPD-DONT-UNDERSTAND

(defstate 
  _gpd-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "Assistant is asking master for whose phone.")
  (:question "- Whose telephone number?")
  (:answer-type :answer)
  (:transitions
   (:always :target _gpd-try-again)
   ))

;;;--------------------------------------------------------------- (GPD) GPD-SORRY

(defstate 
  _gpd-sorry
  (:label "Phone number failure.")
  (:explanation "We don' have the requested number.")
  (:reset)
  (:text "- Sorry, I can't find your number.")
  (:transitions (:failure))
  )

;;;----------------------------------------------------------- (GPD) GPD-TRY-AGAIN

(defstate 
  _gpd-try-again
  (:label "Ask again for phone number.")
  (:explanation "Assistant is asking :CIT-ADDRESS.")
  (:send-message :to :CIT-ADDRESS :self PA_TYLER :action :get-phone-number
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ;; we do not give pattern to allow organizations
                          #|(:pattern . ("personne" ("nom")("prénom")
                                       ("téléphone" ("téléphone"))))
|#)))
  (:transitions
   (:on-failure :target _gpd-sorry)
   (:otherwise
    :exec (omas::assistant-display-text 
           PA_TYLER (moss::make-print-list (read-fact moss::conversation :answer)))
    :success))
  )

;;;================================================================================
;;;
;;;                  PRINT CONCEPT DOCUMENTATION CONVERSATION
;;;
;;;================================================================================

;;; this conversation is meant to print the definition of a concept or properties
;;; to be extracted from a list of words

;;;--------------------------------------------------------------------------------
(eval-when (compile load eval)
  (proclaim '(special 
              _pcd-entry-state
              _pcd-sorry
              )))

(defsubdialog
  _PRINT-CONCEPT-DOCUMENTATION-CONVERSATION
  (:label "Print concept documentation")
  (:explanation 
   "This dialog is meant to print the documentation attached to concepts or ~
    properties referenced by a list of words.")
  )

;;;--------------------------------------------------------------- (PCD) ENTRY-STATE

(defstate 
  _pcd-entry-state
  (:label "start of the PRINT CONCEPT DOCUMENTATION dialog")
  (:entry-state _print-concept-documentation-conversation)
  (:explanation 
   "Initial state may have data left in the input area. The data either represent ~
    an entry point or something else that should be used to select objects.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _pcd-entry-state (conversation input)
  "Using list of words to access objects. If one, OK, if more, must ask user to ~
   select one, if none, failure."
  ;(declare (ignore more-args))
  (let* (entry-point-list object-list)
    (catch :return
      (moss::vformat "_pcd-entry-state /input: ~%  ~S" input)
      ;; first get entry-points
      (setq entry-point-list (moss::find-best-entries input))
      
      ;; if we cannot find anything locally try for a what-is conversation
      (unless entry-point-list
        (throw :return 
               `(:sub-dialog _what-is-conversation :failure ,_pcd-sorry)))
      
      ;; get objects corresponding to entry points
      ;(moss::vformat "_pcd-entry-state /entry-point-list: ~%  ~S" entry-point-list)
      (setq object-list (delete-duplicates (mapcan #'access entry-point-list)))
      (moss::vformat "_pcd-entry-state /object-list: ~%  ~S" object-list)
      
      ;; first filter concepts and properties
      (setq object-list
            (mapcan #'(lambda (xx)
                        (or 
                         (moss::%is-concept? xx)
                         (moss::%is-attribute? xx)
                         (moss::%is-relation? xx)
                         )
                        (list xx))
                         object-list))
      ;(print object-list)
      (if object-list
        ;; print documentation
        (dolist (obj-id object-list)
          (cond
           ((moss::%is-concept? obj-id)
            (send obj-id '=get-documentation :lead "CONCEPT: " 
                  :final-new-line t)
            (send conversation '=display-text *answer*))
           ((moss::%is-attribute? obj-id)
            (send obj-id '=get-documentation :lead "ATTRIBUTE: " 
                  :final-new-line t)
            (send conversation '=display-text *answer*))
           ((moss::%is-relation? obj-id)
            (send obj-id '=get-documentation :lead "RELATION: " 
                  :final-new-line t)
            (send conversation '=display-text *answer*))
           )
          )
        ;; otherwise print failure
        (throw :return 
               `(:transitions ,_pcd-sorry))
        )
      ;; success
      (list :success))))

;;;------------------------------------------------------------------- (PCD) SORRY

(defstate 
  _pcd-sorry
  (:label "Reference search failure.")
  (:explanation "We did not find any explanation for the reference.")
  (:reset)
  (:text "~%- Sorry, I have no concept or property fitting your request.~%" :no-erase)
  (:transitions 
   (:failure))
  )

;;;================================================================================
;;;
;;;                       PRINT HELP CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to help the master by giving information:
;;;   - in general (what tasks are available
;;;      "what can you do for me?" "help." "what can I do?"
;;;   - on a particular topic
;;;      "how do I send a mail?" "help me with the mail?"
;;; specific help corresponding to special help tasks are caught by the task 
;;; task selection mechanism?


(eval-when (compile load eval)
  (proclaim '(special 
              _ph-entry-state  ; required by defstate
              _ph-print-global-help
              )))

;;;--------------------------------------------------- (PH) PRINT-HELP-CONVERSATION

(defsubdialog 
  _print-help-conversation 
  (:label "Help conversation")
  (:explanation 
   "Help ws asked.")
  )

;;;----------------------------------------------------- (PH) PH-PRINT-GLOBAL-HELP

(defstate
  _ph-print-global-help
  (:label "Print general help")
  (:entry-state _print-help-conversation )
  (:explanation "No specific subject was included.")
  (:execute
   (let
     ((obj-id (car (send '>-global-help '=get 'is-title-of)))
      ;(*language* :en)
      (task-list (access '(task)))
      (conversation (omas::conversation TYLER::pa_TYLER))
      )
     (when obj-id (send obj-id '=get-documentation)
           (send conversation '=display-text *answer*))
     (send conversation '=display-text
           "~2%I can perform the following tasks:")
     (dolist (task task-list)
       (send task '=get-documentation :lead "   - ")   
       (send conversation '=display-text *answer*)
       )
     (send conversation '=display-text "~2%")
     ))
  (:transitions
   (:always :success))
  )


;;;================================================================================
;;;
;;;                         PROCESS ASK CONVERSATION
;;;
;;;================================================================================

;;; this conversation is meant to post a message for inspection and send a possible 
;;; answer, or abort the corresponding task

;;; The problem is that the PA has received a request corresponding to the message
;;; being processed and the static part of the skill has posted the message and
;;; is waiting for the answer.
;;; One problem is that there are as many threads as posted messages waiting for an 
;;; answer.

;;; If we went to answer the request message, then the situation is quite complex:
;;; we have several processes:
;;;   - the thread executing the ask skill
;;;   - the converse process
;;;   - the assistant window thread handling the callbacks
;;;   - the timeout thread on the ask thread
;;; One can solve all the problems but the result is not simple.
;;;
;;; A better approach is to consider that all messages ASk and TELL are considered
;;; as INFORM messages, and do not create any specific process (i.e. waiting for a
;;; master's answer), à la e-mail. This allows to send answers directly as INFORM
;;; messages, bypassing the complex mechanism of getting the control back to the
;;; ASK skill.
;;; Thus, all request messages triggering the ASK skill must send a response like
;;; "message well received, transmitted to the master." 

;;; In the simplified approach the role of the PROCESS ASK conversation is to answer
;;; the message when the PROCESS (TRAITER) button is clicked.
;;; The sequence of events is:
;;;   - the ":process-ask" tag is inserted into the agent TO-DO slot
;;;   - the converse process wakes up and triggers the PROCESS ASK dialog
;;;   - a message is posted to the master
;;; If the master wants to answer then
;;;   - the receiver is first determined
;;;   - the *pass-every-char* flag (that should be a specific slot of the interface
;;;     window) is set
;;;   - the master inputs text
;;;   - the DONE (Terminé) button is clicked resetting *pass-every-char* 
;;;   - the text is transferred into the :input area of FACTS
;;;   - the message is sent to the receiver directly as an inform message

(eval-when (compile load eval)
  (proclaim '(special 
              _pa-entry-state  ; required by defstate
              _pa-brush ; clean up lightly
              _pa-destroy ; destroy message return :abort to ask-static
              _pa-discard? ; ask whather we dump the question or not
              _pa-get-answer ; get the answer from the master
              )))

(defsubdialog 
  _process-ask-conversation
  (:label "Process ask dialog")
  (:explanation "Display the content of a question message (ask).")
  )

;;;=========================== service functions ==================================

;;;------------------------------------------------------ (PA) DISPLAY-ASK-MESSAGES

(defun display-ask-message (message conversation)
  "takes a message and displays its content into the assistant pane"
  (let (sender content)
    ;; get sender identity
    (setq sender (omas::from! message))
    ;; get content of the message
    (setq content (car (omas::args message)))
    ;; ask to print it
    (send conversation '=DISPLAY-TEXT 
          (format nil "From : ~A ~%Priority : ~A ~%Object : ~A ~2% ~A"
                  sender
                  (or (cadr (member :priority content)) "Normal")
                  (or (cadr (member :object content)) "")
                  (or (getf content :text) "<empty message>")))))

;;;================================================================================

;;;-------------------------------------------------------------- (PA) ENTRY-STATE
;;; We enter the dialog when the PROCESS (Traiter) button has been clicked. 
;;; FACTS/INPUT contains ":proces-ask" and message is in FACTS/TODO-MESSAGE

(defstate 
  _pa-entry-state
  (:label "process ask dialog")
  (:entry-state _process-ask-conversation)
  (:explanation "Shows the message and ask if an answer is needed.")
  (:execute-preconditions
   ;; the :input area contains a pair (<message> . <process-id>)
   (display-ask-message (read-fact moss::conversation :todo-message)
                        moss::conversation))
  (:question-no-erase "~2%Do you want to answer this message?")
  (:transitions 
   (:yes :target _pa-get-answer) 
   (:otherwise :target _pa-discard?))
  )

;;;-------------------------------------------------------------------- (PA) BRUSH
;;; reset conversation slot
(defstate 
  _pa-brush
  (:label "bookkeeping")
  (:explanation "do some light bookkeeping.")
  (:execute (replace-fact moss::conversation :todo-message nil))
  (:transitions
   (:always :success))
  )

;;;------------------------------------------------------------------ (PA) DESTROY


(defstate 
  _pa-destroy
  (:label "Destroy ASK message.")
  (:explanation "Master does not want to answer the message nor keep it. We ~
                 remove it from the list and return to the task with an :abort~
                 mark, so that it will exit without answering.")
  (:transitions
   (:always
    :exec 
    (let ((agent (car (has-agent moss::conversation))))
      ;; discard the message from the list, assuming it is still selected
      (omas::ASSISTANT-DISCARD-SELECTED-TO-DO-TASK agent)
      ;; clean up conversation
      (replace-fact moss::conversation :todo-message nil))
    :success))
  )

;;;----------------------------------------------------------------- (PA) DISCARD?

(defstate 
  _pa-discard?
  (:label "ask whether we throw the message away.")
  (:explanation "Master did not answer message, ask master if we keep it to it ~
                 later.")
  (:question-no-erase "~2%Do you want to keep the message pour lfor later processing?")
  (:transitions
   (:no :target _pa-destroy)
   (:otherwise :target _pa-brush))
  )

;;;--------------------------------------------------------------- (PA) GET-ANSWER

(defstate
  _pa-get-answer
  (:label "obtain master's answer to an ask message.")
  (:explanation "Record text typed into the master pane until the DONE button is ~
                 pushed. We must disable . and ? terminations.")
  ;; set global so that we can have . and ? in the text
  (:execute-preconditions 
   (let ((win (car (has-input-window moss::conversation))))
     (when win
       (setf (omas::pass-every-char win) t)))
   )
  (:question-no-erase "~% ...Type or tell your answzer then click the end button.") 
  (:answer-analysis)
  )

(defownmethod =answer-analysis _pa-get-answer (conversation input)
  "We process the answer and send it back unless we get an abort.
Arguments:
   conversation: conversation object is
   input: copy of the :input area of FACTS"
  (let (agent message content object)
    (setq agent (car (has-agent conversation)))
    ;(print  `(=====> input ,input))
    ;; if input is different from :abort we build a fuller answer
    (unless (eql input :abort)
      (setq message (read-fact conversation :todo-message)
            content (car (omas::args message))
            object (getf content :object))
      ;; format an answer content adding object to text
      (setq input (list :object (format nil "RE: ~A" (or object " "))
	                    :text (read-fact conversation :raw-input)))
      ;; make an answer and send it back as an INFORM message
      (send-message 
       (make-instance 'omas::message
         :from :TYLER
         :to (omas::from! message)
         :date (get-universal-time)
         :type :inform
         :action :tell
         :args (list input)
         )
       )
      )
    ;; clean FACTS
    (replace-fact conversation :todo-message nil)
    ;; remove message from the tasks to do pane
    (omas::ASSISTANT-DISCARD-SELECTED-TO-DO-TASK agent)
    )
  ;; we then get out of dialog
  '(:success))

;;;================================================================================
;;;
;;;                         PROCESS TELL CONVERSATION
;;;
;;;================================================================================

;;; this conversation is meant to post a message for inspection and quit

(eval-when (compile load eval)
  (proclaim '(special 
              _pt-entry-state  ; required by defstate
              _pt-erase? ; go clean the message
              _pt-clean ; remove item from list
              _pt-brush ; clean up lightly
              )))

(defsubdialog 
  _process-tell-conversation
  (:label "Process tell dialog")
  (:explanation "Display the content of an info message (tell).")
  )

;;;=========================== service functions ==================================

;;;----------------------------------------------------- (PT) DISPLAY-TELL-MESSAGES

(defun display-tell-message (message conversation)
  "takes a message and displays its content into the assistant pane"
  (let (sender content)
    ;; get sender identity
    (setq sender (omas::from! message))
    ;; get content of the message
    (setq content (car (omas::args message)))
    ;; ask to print it
    (send conversation '=DISPLAY-TEXT 
          (format nil "From : ~A ~%Priority : ~A ~%Object : ~A ~2% ~A"
                  sender
                  (or (cadr (member :priority content)) "Normal")
                  (or (cadr (member :object content)) "")
                  (or (getf content :text) "<empty message>")))))

;;;=========================================================?======================
;;;                        PROCESS TELL CONVERSATION STATES
;;;================================================================================

;;;-------------------------------------------------------------- (PT) ENTRY-STATE

(defstate 
  _pt-entry-state
  (:label "Process tell dialog")
  (:entry-state _process-tell-conversation)
  (:explanation "Affiche un message et demande si on le garde.")
  (:execute (display-tell-message 
             (car (read-fact moss::conversation :message)) MOSS::Conversation))
  (:question-no-erase "~2%OK? Do I erase the message?")
  (:transitions 
   (:yes :target _pt-clean) 
   (:otherwise :target _pt-brush))
  )

;;;-------------------------------------------------------------------- (PT) BRUSH

(defstate 
  _pt-brush
  (:label "bookkeeping")
  (:explanation "remove item from facts.")
  (:execute (replace-fact moss::conversation :message nil))
  (:transitions
   (:always :success))
  )

;;;-------------------------------------------------------------------- (PT) CLEAN

(defstate 
  _pt-clean
  (:label "Cleaning state")
  (:explanation "Ask whether we keep the message in the to-do pane.")
  (:execute (omas::ASSISTANT-DISCARD-SELECTED-TO-DO-TASK 
             (car (has-agent moss::conversation)))
            (replace-fact moss::conversation :message nil))
  (:transitions
   (:always :success))
  )

  
;;;================================================================================
;;;
;;;                      PUBLISH NEWS CONVERSATION
;;;
;;;================================================================================

(eval-when (compile load eval)
  (proclaim '(special 
              _publish-new-conversation; required by defstate
              _publish-new-entry-state
              _pnd-get-category
              _pnd-get-content
			  _pnd-get-sections
              _pnd-publish
              _pnd-sorry)))

;;;------------------------------------------------ (PND) PUBLISH-NEW-CONVERSATION

(defsubdialog 
  _publish-new-conversation
  (:label "Publish new conversation")
  (:explanation 
   "Master is trying to publish a new.")
  )

(defstate 
  _publish-new-entry-state
  (:entry-state _publish-new-conversation)
  (:label "Publish new dialog entry")
  (:explanation "Ask for the title of the new.")  
  (:question-no-erase "~%- Object?")
  (:answer-type :answer)
  (:execute 
    (omas::remember PA_TYLER (moss::read-fact moss::conversation :input)
     	:news-title))
  (:transitions
   (:always  :target _pnd-get-category)))

;;;------------------------------------------------------------ (PND) GET-CATEGORY

(defstate 
  _pnd-get-category
  (:label "Ask master the content.")
  (:explanation "Obtain title, ask for category")
  ;(format t "~S~%" (moss::read-fact moss::conversation :input))
  ;(format t "~% TITULO A GUARDAR: ~S" (moss::read-fact moss::conversation :input))
  ;; Store title in agent´s short term memory  OMARAGP PORQUE ESTO NO FUNCIONA  
  (:question-no-erase "~%- Under which topic do you want to publish the news?")
  (:execute 
    (omas::remember PA_TYLER (moss::read-fact moss::conversation :input)
     	:news-section))
  (:transitions
   (:patterns ((?* ?x) "topics" (?* ?y))
        :target _pnd-get-sections)
   (:otherwise  :save :target _pnd-get-content)))

;;;------------------------------------------------------------- (PND) GET-CONTENT

(defstate 
  _pnd-get-content
  (:label "Ask master the content.")
  (:explanation "Obtain category, ask for content")
  
  (:execute-preconditions 
   (setf (omas::pass-every-char (car (has-input-window moss::conversation))) t)
   ;; include :verbatim flag into the FACT base (?)
   (replace-fact moss::conversation :verbatim t)
   )
  (:question-no-erase 
    "~%- Please give the content of the news, then click the DONE button.")
  (:answer-type :answer)
  (:execute 
    ;; Store category in agent´s short term memory						   
    (remember PA_TYLER (moss::read-fact moss::conversation :input)
     	 :news-content))
  (:transitions
   (:always  :target _pnd-publish)))

;;;------------------------------------------------------------ (PND) GET-SECTIONS

(defstate 
  _pnd-get-sections
  (:label "Ask NEWS about sections.")
  (:explanation "Master does not know sections. Asking NEWS for the list.")
  (:send-message :to :NEWS :self PA_TYLER :action :get-all-sections
                 :args '((:language . :en)))
  (:execute 
    (omas::assistant-display-text PA_TYLER
	  (format nil "~S" (moss::read-fact moss::conversation :answer))))
  (:transitions
   (:on-failure :target _pnd-sorry)
   (:otherwise
	:target _pnd-get-category)))

;;;----------------------------------------------------------------- (PND) PUBLISH

(defstate 
  _pnd-publish
  (:label "Ask master the content.")
  (:explanation "Ask NEWS to create the news")
  (:send-message :to :NEWS :self PA_TYLER :action :publish-new
                 ;; Recall gets data stored un ALBERT´s short term memory
                 :args `(((:title . ,(recall PA_TYLER :news-title))
                          (:section . ,(recall PA_TYLER :news-section))
                          (:content . ,(recall PA_TYLER :news-content))
						  (:author  "Tyler")
                          (:language . :en))))
  (:execute (print (car (read-fact moss::conversation :input))))
  (:transitions
     (:on-failure :target _pnd-sorry)
	 (:otherwise :success))
  )

;;;---------------------------------------------------------------- (PND) PND-SORRY

(defstate 
  _pnd-sorry
  (:label "Publish failure.")
  (:explanation "It was an error while publishing the new.")
  (:text "Sorry, the NEWS service agent could not publish your text.~%")
  (:reset)
  (:transitions (:failure))
  )

;;;================================================================================
;;;
;;;                       SET FONT SIZE CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to modify the size of the dialog font, usually to
;;; increase it or to return to the default value.

(eval-when (compile load eval)
  (proclaim '(special 
              _sfs-entry-state  ; required by defstate
              _sfs-dont-understand
              _sfs-try-again
              _sfs-sorry)))

;;;-------------------------------------------------- (SFS) FONT-SIZE-CONVERSATION

(defsubdialog 
  _set-font-size-conversation
  (:short-name sfs)
  (:label "Font size conversation")
  (:explanation 
   "The data is searched for the presence of words like increase, ~
    bigger, smaller, defaults, reset,...")
  ;(:action _sfs-action)
  )

;;;--------------------------------------------------- (SFS) FONT-SIZE-ENTRY-STATE

(defstate 
  _sfs-entry-state
  (:entry-state _set-font-size-conversation)
  (:label "Font size dialog entry")
  (:explanation "The data is searched for the presence of words like increase, ~
                 bigger, defaults, reset,...")
  (:transitions
   (:patterns (((?* ?x) "increase" (?* ?y))
               ((?* ?x) "bigger" (?* ?y))
               ((?* ?x) "larger" (?* ?y)))
              :exec (omas::assistant-use-larger-font PA_TYLER) :success)
   (:patterns (((?* ?x) "decrease" (?* ?y))
               ((?* ?x) "smaller" (?* ?y)))
              :exec (omas::assistant-reset-font PA_TYLER) :success)
   (:patterns (((?* ?x) "use" "default" (?* ?y))
               ((?* ?x) "reset" (?* ?y))
               ((?* ?x) "defaults" (?* ?y)))
              :exec (omas::assistant-reset-font PA_TYLER) :success)
   (:otherwise :target _sfs-dont-understand)
   )
  )

;;; add entry state to dialog header
;(send 'moss::$ESTS '=delete-all _set-font-size-conversation)
;(moss::%link _set-font-size-conversation 'moss::$ESTS _sfs-entry-state)

;;;--------------------------------------------------------- (SFS) DONT-UNDERSTAND

(defstate 
  _sfs-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "Assistant is asking master for font-size clarification.")
  (:question "~%- Do you want a bigger font size, a smaller one, or use the default?")
  (:transitions
   (:always :target _sfs-try-again)))

;;;------------------------------------------------------------------- (sfs) SORRY

(defstate 
  _sfs-sorry
  (:label "Font dialog failure.")
  (:explanation "We cannot make things out from the data.")
  (:reset)
  (:answer "~%- Sorry I do not understand what you want me to do with the font?.~%")
  (:transitions
   (:always :failure))
  )

;;;--------------------------------------------------------------- (sfs) TRY-AGAIN

;;; The simplest use is to take text in the HAS-DATA slot of the Q-CONTEXT
;;; and send a message to :SA-ADDRESS

(defstate 
  _sfs-try-again
  (:label "Font size try again")
  (:explanation "we asked master and try to pick up the answer.")
  (:transitions
   (:patterns (((?* ?x) "bigger" (?* ?y))
               ((?* ?x) "larger" (?* ?y)))
              :exec (omas::assistant-use-larger-font PA_TYLER) :success)
   (:patterns (((?* ?x) "smaller" (?* ?y)))
              :exec (omas::assistant-reset-font PA_TYLER) :success)
   (:patterns (((?* ?x) "default" (?* ?y))
               ((?* ?x) "defaults" (?* ?y)))
              :exec (omas::assistant-reset-font PA_TYLER) :success)
   (:otherwise :target _sfs-sorry)
   )
  )

;;;================================================================================
;;;
;;;                           TELL CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to tell something to a remote PA.

;;; We must first locate our receiver (presumably the PA of somebody in the coterie
;;; We then want to prepare topic and text, the send the message

(eval-when (compile load eval)
  (proclaim '(special 
              _tell-entry-state  ; required by defstate 
			  _tell-ask-destination
              _tell-choose-destination 
              _tell-get-data
              _tell-get-destination
			  _tell-process-destination
              _tell-send-message
              _tell-sorry
              )))

;;;================================================================================
;;;                       TELL CONVERSATION SERVICE FUNCTIONS
;;;================================================================================

;;;------------------------------------------------------------ EXTRACT-DESTINATION
;;; See ASK dialog
;;;================================================================================
;;;                            TELL CONVERSATION STATES
;;;================================================================================

;;;------------------------------------------------------- (TELL) TELL-CONVERSATION

(defsubdialog 
  _tell-conversation 
  (:label "Tell conversation")
  (:explanation 
   "We want our PA to tell something to somebody.")
  )

;;;-------------------------------------------------------- (TELL) TELL-ENTRY-STATE

(defstate 
  _tell-entry-state
  (:label "Tell dialog entry")
  (:entry-state _tell-conversation)
  (:explanation "We try to get destination from the input data.")
  (:transitions
    ;; normally the following test never happens
    (:test (null (read-fact moss::conversation :input)) 
	        :target _tell-ask-destination)
	(:patterns
	  (((?* ?X) "tell" (?* ?Y)) 
       ((?* ?X) "inform" (?* ?Y))
       ((?* ?X) "let" (?* ?Y) "know" (?* ?Z))
	   ((?* ?x) "send" (?* ?Z) "to" (?* ?Y))
	   ((?* ?X) "message" "for" (?* ?Y))
	   ((?* ?X) "message" "to" (?* ?Y))
	   )
	  :keep ?Y
	  :target _tell-process-destination)
	(:otherwise :target _tell-ask-destination))
	)
	  
;;;--------------------------------------------------------- (TELL) ASK-DESTINATION

(defstate
  _tell-ask-destination
  (:label "Tell ask destination")
  (:explanation "ask to whom we send message")
  (:question-no-erase 
   "~% To whom do you want to send the message?")
  (:transitions (:always :target _tell-process-destination))
  )

;;;------------------------------------------------------ (TELL) CHOOSE-DESTINATION

(defstate 
  _tell-choose-destination
  (:label "Tell choose destination")
  (:explanation "ambiguous destination (list) select one.")
  (:execute
   (let* ((conversation moss::conversation)
          (choices (read-fact conversation :to-choose))
          )
    ;; remove the temporary data
    (replace-fact conversation :to-choose nil)
    (replace-fact conversation :to (car choices))))
  (:transitions (:always :target _tell-get-data)))

;;;--------------------------------------------------------------- (TELL) GET-DATA

(defstate 
  _tell-get-data
  (:label "set up parameters for continuous input.")
  (:explanation "Let the master type text into the pane until the DONE button is ~
                 pushed. We must disable . and ? terminations.")
  ;; set global flag in the interface window so that we can have . and ? in the text
  (:execute-preconditions 
   (setf (omas::pass-every-char (car (has-input-window moss::conversation))) t)
   ;; include :verbatim flag into the FACT base (?)
   (replace-fact moss::conversation :verbatim t)
   )
  (:question-no-erase 
   "~% ...Type or tell your text, then click the DONE button.")
  (:transitions
   (:always :target _tell-send-message))
  )

  #|
;;;--------------------------------------------------------- (TELL) GET-DESTINATION

(defstate 
  _tell-get-destination
  (:label "obtain master's destination for message.")
  (:explanation "Get the id of the personal assistant of the receiver.")
  ;; set global so that we can have . and ? in the text
  (:question-no-erase 
   "~% To whom do you want to send the message?") 
  (:execute (replace-fact moss::conversation :to
                 (extract-destination (read-fact moss::conversation :input))))
  (:transitions
    (:test (null (read-fact moss::conversation :to)) :target _tell-sorry)
	(:test (listp (read-fact moss::conversation :to)) 
	    :exec (replace-fact moss::conversation :to-choose destination)
		:target _tell-choose-destination)
	(:otherwise
		:target _tell-get-data))
  )
  

(defstate 
  _tell-get-destination
  (:label "obtain master's destination for message.")
  (:explanation "Get the id of the personal assistant of the receiver.")
  ;; set global so that we can have . and ? in the text
  (:question-no-erase 
   "~% To whom do you want to send the message?") 
  (:answer-analysis)
  )

(defownmethod =answer-analysis _tell-get-destination (conversation input)
  "We process the answer and record it unless we get an abort.
Arguments:
   agent: assistant."
  (let (destination)
    ;; analyze input
    (setq destination (extract-destination input))
    
    (cond
     ((null destination)
      (list :transition _tell-sorry))
     ((listp destination)
      (replace-fact conversation :to-choose destination)
      (list :transition _tell-choose-destination))
     (t
      (replace-fact conversation :to destination)
      ;; we then get out of dialog
      (list :transition _tell-get-data)))))	
|#
;;;----------------------------------------------------- (TELL) PROCESS-DESTINATION

(defstate 
  _tell-process-destination
  (:label "Tell: Process destination")
  (:explanation "process input to find to whom we send message")
  (:execute (replace-fact moss::conversation :to 
                 (extract-destination (read-fact moss::conversation :input))))
  (:transitions
    (:test (null (read-fact moss::conversation :to)) :target _tell-sorry)
	(:test (listp (read-fact moss::conversation :to)) 
	    :exec (replace-fact moss::conversation 
		                    :to-choose (read-fact moss::conversation :to))
		:target _tell-choose-destination)
	(:otherwise
		:target _tell-get-data))
  )
  
;;;------------------------------------------------------------ (TELL) SEND-MESSAGE

(defstate 
  _tell-send-message
  (:label "Tell send final message")
  (:explanation "build and send the message.")
  (:send-message-no-wait 
   :to (pop-fact moss::conversation :to)
   :type :inform
   :action :tell
   :args (list (list :text (pop-fact moss::conversation :raw-input))))
  (:text "~&...Message was sent")
  (:transitions (:always :success)) ; return
  )

;;;------------------------------------------------------------------- (TELL) SORRY

(defstate 
  _tell-sorry
  (:label "Tell failure.")
  (:explanation "We don't have the needed parameters")
  (:text 
   "Sorry, I don't see to whom you want to send a message.~%")
  (:transitions (:always :failure))
  )

;;;================================================================================
;;;
;;;                       TRACE CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to modify the size of the dialog font, usually to
;;; increase it or to return to the default value.

(eval-when (compile load eval)
  (proclaim '(special 
              _tr-entry-state
              )))

;;;------------------------------------------------------- (TR) TRACE-CONVERSATION

(defsubdialog 
  _trace-conversation
  (:label "Trace conversation")
  (:explanation 
   "Command to toggle the dialog trace."))

;;;-------------------------------------------------------- (TR) TRACE-ENTRY-STATE

(defstate 
  _tr-entry-state
  (:label "Trace dialog entry")
  (:entry-state _trace-conversation)
  (:explanation "We search the input for the trace command.")
  
  (:transitions
   ;; system commands
   (:patterns (((?* ?x) "show" "transitions" (?* ?y))
               ((?* ?x) "trace" "transitions" (?* ?y))
               ((?* ?x) "transitions" "on" (?* ?y))
               ((?* ?x) "print" "transitions" (?* ?y)))
              :exec (setq moss::*transition-verbose* t
                          moss::*traced-agent* PA_TYLER) :success)
   (:patterns (((?* ?x) "no" "trace" (?* ?y))
               ((?* ?x) "kill" "trace" (?* ?y))
               ((?* ?x) "trace" "off" (?* ?y))
               ((?* ?x) "stop" "trace" (?* ?y))
               ((?* ?x) "no" "verbose" (?* ?y)))
              :exec (setq moss::*verbose* nil
                          moss::*transition-verbose* nil
                          moss::*traced-agent* nil) :success)
   (:patterns (((?* ?x) "trace" "on" (?* ?y))
               ((?* ?x) "set" "trace" (?* ?y))
               ((?* ?x) "start" "trace" (?* ?y))
               ((?* ?x) "verbose" (?* ?y)))
              :exec (setq moss::*verbose* t
                          moss::*traced-agent* PA_TYLER) :success)
   (:patterns (((?* ?x) "kill" "transitions" (?* ?y))
               ((?* ?x) "hide" "transitions" (?* ?y))
               ((?* ?x) "transitions" "off" (?* ?y))
               ((?* ?x) "do" "not" "show" "transitions" (?* ?y)))
              :exec (setq moss::*transition-verbose* nil) :success)
   (:otherwise :failure)
   )
  )

#|
;;; add entry state to dialog header
(send 'moss::$ESTS '=delete-all _trace-conversation)
(moss::%link _trace-conversation 'moss::$ESTS _tr-entry-state)
|#

;;;================================================================================
;;;
;;;                      WHAT-IS CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to obtain and print information about something
;;; corresponding to unformatted data

;;;(defParameter *empty-words* 
;;;  '("the" "a" "an" "of" "from" "to" "those" "these" "that" "s" "?"
;;;    "le" "la" "les" "de" "du" "des" #+MCL "à" #+MICROSOFT-32 "à" "aux" 
;;;    "ce" "ces"))

;;;================================================================================

(eval-when (compile load eval)
  (proclaim '(special 
              _what-is-conversation  ; required by defstate
              _wai-entry-state
              _wai-ask-staff
              _wai-get-info
              _wai-process-staff-answer
              _wai-sorry)))

(defsubdialog 
  _what-is-conversation
  (:label "Get info dialog")
  (:explanation "Master is trying to get info about some data.")
  )

;;;------------------------------------------------------------- (WAI) ENTRY-STATE

(defstate 
  _wai-entry-state
  (:label "What is conversation")
  (:entry-state _what-is-conversation)
  (:explanation "Assistant is trying to get object info from its knowledge base.")
  (:transitions
   ;;keep only relevant data
   (:patterns (((?* ?x) "what" "is" (?* ?y) "?")
               ((?* ?y) "is" "what" "?" (?* ?x))
               ((?* ?y))  ; when entering from print-concept-documentation
               )
              :keep ?y
              :target _wai-get-info)
   (:otherwise :target _wai-sorry)
   )
  )

;;;--------------------------------------------------------------- (WAI) ASK-STAFF
;;; changing strategy to :collect-answers
;;; rather than sending a broadcast right away, maybe we should
;;; 1. look into local KB
;;; 2. look into our agenda (sa-address)
;;; 3. ask BIBLIO
;;; This would avoid strange answers resulting from misinterpretation of the
;;; query.

(defstate 
  _wai-ask-staff
  (:label "Ask staff.")
  (:explanation "Ask the staff for help.")
  (:send-message :to :all :self PA_TYLER :action :what-is :timeout 2
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 :strategy :collect-answers
                 )
  (:transitions 
   (:always :target _wai-process-staff-answer))
  )

;;;---------------------------------------------------------------- (WAI) GET-INFO

(defstate 
  _wai-get-info
  (:label "Looking into local KB info.")
  (:explanation "Look into own KB to get info.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _wai-get-info (conversation input)
  "we try first to get info locally and if it fails ask all agents.
Arguments:
   agent: assistant."
  (declare (special *empty-words*))
  (let* (entry-list)
    (catch :return
      ;; clean input of parasitic words
      (setq input (mapcan #'(lambda (xx) 
                              (if (member xx *empty-words* :test #'string-equal)
                                nil
                                (list xx)))
                          input))
      (moss::vformat "TYLER: _wai-get-info /+++ cleaned input:~& ~S" input)
      ;; if nothing left, then go ask staff
      (unless input
        (throw :return `(:transition ,_wai-ask-staff)))
      ;; check whether the resulting words correspond to entry points
      (setq entry-list (moss::find-best-entries input))
      (moss::vformat "TYLER: _wai-get-info /entry list:~&    ~S" entry-list)
      ;; if nothing left go ask staff
      (unless entry-list 
        (throw :return `(:transition ,_wai-ask-staff)))
      ;; if more than one result, ask for help (not yet)
      ;; get documentation or CV
      ;; print it
      (dolist (item entry-list)
        (send item '=get-documentation :lead "   - ")   
        (send conversation '=display-text *answer*)
        )
      ;; transfer to :success    
      (list :success) 
      )
    ))

;;;------------------------------------------------------ (WAI) PROCESS-STAFF-ANSWER

(defstate 
  _wai-process-staff-answer
  (:label "Process staff answer.")
  (:explanation "Process anwers from the staff agents.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _wai-process-staff-answer (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   agent: assistant."
  (declare (ignore input))
  (let* ((agent (car (has-moss-agent conversation)))
         (answer-list (omas::answer agent))
         (*language* :en)
         )
    ;(format *debug-io* "~&+++ =resume; HAS-TO-DO: ~S" (has-to-do conversation))
    ;; clean up HAS-TO-DO (contains "answer-there")
    ;(setf (has-to-do conversation) nil)
    ;; answe? contains either the word "failure" or a list of lists of strings
    ;; (("Jean-Paul Barthes is a professor at UTC ..."
    ;;  "Dominique Barthes-Biesel is a professor at UTC ...")
    ;;  ("Jean-Paul Barthes has NN publications on:  ...")
    ;;  ...)
    ;; 
    (moss::vformat "~&>>> WAIdialog /Answer to the send-request:~% ~S" answer-list)
    ;; failure if :failure is returned or :error on timeout
    (cond
     ((or (equal "failure" answer-list) (eql :error answer-list))
      `(:transition ,_wai-sorry))
     ((null answer-list)
      (send conversation '=display-text "I don't know...")
      (list :success))
     (t 
      (moss::vformat " _wai-ask-staff /answer-list: ~& ~S" answer-list)
      (send conversation '=display-text "It is:")
      ;; each item of the answer list is like 
      ;;    ("personne" ("nom" "...")("prénom" "..."))
      (dolist (answer answer-list)
        ;; answer could be "failure" rather than a list of strings
        (unless (and (stringp answer) (string-equal answer "failure"))
          (dolist (item answer)
            (send conversation '=display-text 
                  (concatenate 'string "~%" (car item)))
            (send conversation '=display-text
                  (format nil ": ~{~%     ~{~A~^, ~}~}" (cdr item))))))
      (list :success)))
    ))

;;;------------------------------------------------------------------- (WAI) SORRY

(defstate 
  _wai-sorry
  (:label "Failure to get info.")
  (:explanation "We could not find anything.")
  (:reset)
  (:text "Sorry, I could not find information for the data.")
  (:transitions 
   (:failure))
  )


;;;================================================================================
;;;
;;;                    WHO IS (GET PERSON INFO) CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to print information about a person.

;;;------------------------------------------------------------ CHECK-ENTRY-POINTS

(defun check-entry-points (input)
  "tries to get entry points by combining words.
Arguments:
   input: list of one or more words
Return:
   nil or a list like 
     (\"barthes\" ($T-PERSON-NAME.OF . $E-PERSON.1) ($T-PERSON-NAME.OF . $E-PERSON.2))"
  (let (result entry entry-list expr-list)
    ;; combine all possible words
    (setq expr-list (make-word-combinations input))
    ;; try all possibilities
    (dolist (expr expr-list)
      ;; entry point ?
      (when (setq entry (moss::%entry? expr))
        ;; extract corresponding objects
        (setq result (moss::%%get-objects-from-entry-point entry))
        ;; return to the executing environment
        (moss::vformat "...trying ~S in package ~S as entry...~&   => ~S" 
                       expr *package* result)
        (if result (setq entry-list (append entry-list result)))
        ))
    ;; clean list, eliminating tasks whose performative is different of the required
    ;; one
    (setq entry-list (delete-duplicates entry-list :test #'equal))
    (moss::vformat "check-entry-points /Resulting entry list ~&  => ~S" entry-list)
    (mapcar #'cdr entry-list)))

#|
? (TYLER::check-entry-points '("telephone" "barthes" ))
;***** ...trying "telephone" in package #<Package "SA-ADDRESS"> as entry...
   => ((MOSS::$ENAM.OF . $E-PHONE))
;***** ...trying "barthes" in package #<Package "SA-ADDRESS"> as entry...
   => (($T-PERSON-NAME.OF . $E-PERSON.2) ($T-PERSON-NAME.OF . $E-PERSON.3)
       ($T-PERSON-NAME.OF . $E-PERSON.4))
;***** check-entry-points /Resulting entry list 
  => ((MOSS::$ENAM.OF . $E-PHONE) ($T-PERSON-NAME.OF . $E-PERSON.2)
      ($T-PERSON-NAME.OF . $E-PERSON.3) ($T-PERSON-NAME.OF . $E-PERSON.4))
($E-PHONE $E-PERSON.2 $E-PERSON.3 $E-PERSON.4)
|#
;;;-------------------------------------------------------- MAKE-WORD-COMBINATIONS

(defun make-word-combinations (word-list)
  ;; when empty list or only one word return
  (unless (cdr word-list) (return-from make-word-combinations word-list))
  ;; otherwise build word combinations
  (let ((input-length (length word-list)) ; e.g. 5
        result)
    (dotimes (nn (1+ input-length)) ; e.g. nn: 0 -> 4
      ;; build cursor to move through the list
      (dotimes (jj nn) ; e.g. jj: 0, then jj: 0->1
        (push
         (apply #'moss::%make-phrase 
                (subseq word-list jj (+ (- input-length nn) jj 1)))
         result)))
    (reverse result)))

#|
? (make-word-combinations '("a" "b" "c" "d"))
("a b c d" "a b c" "b c d" "a b" "b c" "c d" "a" "b" "c" "d")
(moss::%%make-name "a b c" :plain)
A-B-C
|#
;;;================================================================================

(eval-when (compile load eval)
  (proclaim '(special 
              _get-person-info-conversation  ; required by defstate
              _gpi-entry-state
              _gpi-ask-staff
              _gpi-get-info
              _gpi-sorry)))

(defsubdialog 
  _get-person-info-conversation
  (:label "Get person info dialog")
  (:explanation "Master is trying to obtain a telephone number.")
  )

(defstate 
  _gpi-entry-state
  (:label "Get person info conversation")
  (:entry-state _get-person-info-conversation)
  (:explanation "Assistant is trying to get the info from its knowledge base.")
  (:transitions
   ;;keep only relevant data
   (:patterns (((?* ?x) "who" "is" (?* ?y) "?")
               ((?* ?x) "is" "who" (?* ?y))
               )
              :keep ?y
              :target _gpi-get-info)
   (:otherwise :target _gpi-sorry)
   )
  )

;;;--------------------------------------------------------------- (WOI) ASK-STAFF
;;; changing strategy to :collect-answers
;;; rather than sending a broadcast right away, maybe we should
;;; 1. look into local KB
;;; 2. look into our agenda (ADDRESS)
;;; 3. ask BIBLIO
;;; This would avoid strange answers resulting from misinterpretation of the
;;; query.

(defstate 
  _gpi-ask-staff
  (:label "Ask other agents for help.")
  (:explanation "Ask the staff for help.")
  (:send-message :to :all :self PA_TYLER :action :who-is :timeout 4
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 :strategy :collect-answers
                 )
  (:transitions
   ;; can't use :on-failure that test the FACTS/input area
   ;; answers to messages are put into the FACTS/answer area
   (:test (null (read-fact moss::conversation :answer)) :target _gpi-sorry)
   (:otherwise
    :exec (dolist (answer (read-fact moss::conversation :answer))
              (dolist (item answer)
                (send moss::conversation '=display-text item)))
    :success))
  )

;;;---------------------------------------------------------------- (WOI) GET-INFO

(defstate 
  _gpi-get-info
  (:label "Looking into local KB info.")
  (:explanation "Look into own KB to get answer to who-is?.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _gpi-get-info (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   conversation: the conversation object
   input: a copy of the FACTS/input area"
  (let* (entry-list)
    (catch :return
      ;; clean input of parasitic words
      (setq input (mapcan #'(lambda (xx) 
                              (if (member xx *empty-words* :test #'string-equal)
                                nil
                                (list xx)))
                          input))
      (moss::vformat "TYLER: _gpi-get-info /+++ cleaned input:~& ~S" input)
      ;; if nothing left, then ask staff
      (unless input
        (throw :return `(:transition ,_gpi-ask-staff)))
      ; check whether the resulting words correspond to entry points
      (setq entry-list (check-entry-points input))
      (moss::vformat "TYLER: _gpi-get-info /entry list:~& ~S" entry-list)
      ;; if nothing left go ask staff
      (unless entry-list 
        (throw :return `(:transition ,_gpi-ask-staff)))
      ;; if more than one person ask for help (not yet)
      ;; get documentation or CV
      ;; print it
      (dolist (item entry-list)
        (send item '=get-documentation)
        (send conversation '=display-text *answer*))
      ;; transfer to :success    
      (list :success) 
      )
    ))

;;;------------------------------------------------------------------- (WOI) SORRY

(defstate 
  _gpi-sorry
  (:label "Sorry: no info.")
  (:explanation "We could not find anything.")
  (:reset)
  (:text ("~%- Sorry, I have not the slightest idea. Please check the spelling.~%"
          "- I don't know. Please check the spelling.~%"))
  (:transitions 
   (:failure))
  )

;;;================================================================================
;;;================================================================================

;;;================================================================================
;;;
;;;                  CONVERSATIONS AND SUB-CONVERSATIONS
;;;
;;;================================================================================

;;; we declare the top-level conversations
;;; the entry point of the dialog is _MAIN-CONVERSATION

(eval-when (:load-toplevel :compile-toplevel :execute)
  (proclaim '(special 
              _MAIN-CONVERSATION
              _PROCESS-CONVERSATION
              )))

;;;================================================================================
;;;================================================================================
;;;
;;;                       MAIN CONVERSATION (CONTROL LOOP)
;;;
;;;================================================================================
;;;================================================================================

;;; create a dialog header to be used for the main conversation

(defindividual 
  MOSS-DIALOG-HEADER
  (HAS-MOSS-LABEL "Main conversation loop")
  (HAS-MOSS-EXPLANATION "This is the main conversation loop.")
  (:var _main-conversation))

;; declare that conversation, replacing default conversation in the agent structure
(setf (omas::dialog-header PA_TYLER) _main-conversation)

;;; debugging function
(defun cc () (moss::start-conversation _main-conversation))

;;;===== states are declared as global variables
;;; the set of states can be considered as a plan to be executed for conducting the
;;; conversation with the master

;;;--------------------------------------------------------------------------------
(eval-when (:compile-toplevel :load-toplevel :execute)
  (proclaim '(special 
              _mc-entry-state
              _mc-get-input
              _mc-more?
              _mc-process
              _mc-sleep)))
;;;--------------------------------------------------------------------------------

;;;--------------------------------------------------------------- (MC) ENTRY-STATE
;;; ENTRY STATE
;(defun ff () (start-conversation _main-conversation :first-time t))
;(defun gg () (start-conversation _main-conversation))

;;; define an entry instance and record it in a global variable.
;;; ****** we should enter a frame on the conversation frame list, to handle
;;; success or failure returns from sub-dialogs. I.e. we should make a transition
;;; on the success or failure state of the main conversation.
;;; entry (_mc-failure _mc-success _mc-entry-state nil) ; no goal 

;;; Action:
;;;   =EXECUTE
;;;   - get ACTION if any from the dialog header
;;;   - reset conversation: HAS-MOSS-FACTS, HAS-MOSS-TASK, HAS-MOSS-TASK-LIST
;;;   - if ACTION IS non NIL, set conversation HAS-GOAL with a pattern obtaines from
;;;        sending ACTION a =create-pattern message
;;;     otherwise, reset conversation HAS-GOAL
;;;   - display welcome message
;;;
;;;   <Master enters text>
;;;
;;;   =RESUME
;;;   - if INPUT contains one of the abort commands, throws to :DIALOG-ERROR
;;;   - makes a transition to _MC-PROCESS

(defstate _mc-entry-state
  (:entry-state _main-conversation)
  (:label "Début du dialogue")
  (:explanation 
   "Initial state when the assistant starts a conversation. Send a ~
    welcome message and wait for data. Also entered on a restart ~
	following an abort.")
  (:reset-conversation)
  (:execute-preconditions 
    (send moss::conversation '=clean-output-pane))
  (:text 
   (:fr "Attention! Ce dialogue est trés limité. Chaque phrase est ~
    indépendante et l'on ne peut utiliser des pronoms référents."
	:en "Warning. This is not a real dialog in the sense that every ~
         question usually does not use the previous context. Thus, ~
		 referents like pronouns or ellipsis (referring to part of ~
		 the answer) won't work."))
  (:question-no-erase
   ("Hello! What can I do for you?"
    "Hi! What would you like to know?"
    "Hi! I will try to do my best to answer your question or ~
     execute your orders, but remember that my IQ is limited."))
  (:transitions 
   (:always :target _mc-process))
  )

;;;----------------------------------------------------------------- (MC) GET-INPUT

(defstate _mc-get-input
  (:label "Get input")
  (:explanation
   "State in which the user inputs his request.")
  (:reset)
  (:question 
    (:fr "Je vous écoute...." 
	 :en "I am listening..."))
  (:transitions
   (:always :target _mc-process)))

;;;-------------------------------------------------------------------- (MC) MORE?
;;; Action:
;;;   - If answer is "rien", "non" transition to SLEEP
;;;   - if answer is "oui" transition to GET-INPUT
;;;   - anything else sends to PROCESS

(defstate _mc-more?
  (:label "More?")
  (:explanation "Asking the user it he wants to do more interaction.")
  (:reset-conversation)
  (:question-no-erase
   ((:fr "Que puis-je faire d'autre pour vous ?"
     :en "What else can I do for you?")
    (:fr "Y a-t-il autre chose que je puisse faire pour vous ?"
	 :en "Is there anything else I could do for you?")
    (:fr "Avez-vous d'autres questions ?"
	 :en "Any other question?")
    (:fr "OK?"
	 :en "OK?"))
   )
  (:transitions 
   (:starts-with ("rien" "nothing") :target _mc-sleep)
   (:no :target _mc-sleep)
   (:yes :target _mc-get-input)
   (:otherwise :target _mc-process)))

;;;------------------------------------------------------------------- (MC) PROCESS

(defstate _mc-process
  (:label "Process state of main conversation")
  (:explanation
   "Here we call the main process organized as a sub-conversation.")
  (:transitions
   (:always :sub-dialog _process-conversation
            :success _mc-more?
            :failure _mc-more?)))

;;;--------------------------------------------------------------------- (MC) SLEEP

(defstate _mc-SLEEP
  (:label "Nothing to do")
  (:explanation
   "User said she wanted nothing more.")
  (:reset)
  (:text 
   (:fr "OK. J'attends que vous soyez prêt."
    :en "Fine. I wait until you are ready."))
  (:question-no-erase 
   (:fr "Réveillez moi en tapant ou en disant quelque chose..."
    :en "Wake me up by typing something..."))
  (:transitions
   (:always :target _mc-process)))

;;;================================================================================
;;;
;;;                      MAIN CONVERSATION (EXECUTION PART)
;;;
;;;================================================================================

;;; this conversation is intended to process the input from te user by first
;;; determining the performative, then the list of possible tasks

(defsubdialog 
  _process-conversation 
  (:label "Process conversation")
  (:explanation 
   "Processing steps of the main conversation.")
  (:states _mc-eliza
           _mc-find-performative
           _mc-entry-state
           _mc-failure
           _mc-find-performative
           _mc-find-task
           _mc-select-task
           _mc-task-dialog)
  )

;;;===============================================================================
;;;                            Service functions
;;;===============================================================================

;;;-------------------------------------------------------------------- PRINT-TIME

(defun print-time (conversation &optional (language *language*))
  (multiple-value-bind (second minute hour date month year)
                       (decode-universal-time (get-universal-time))
    (send conversation '=display-text
          (case language
            (:en
             (format nil "Current time is ~S:~S:~S and today is the ~S/~S/~S"
                     hour minute second date month year))
            (:fr
             (format nil 
                     "Il est actuellement ~S:~S:~S et nous sommes aujourd'hui le ~
                      ~S/~S/~S"
                     hour minute second date month year)))
          )))

;;;===============================================================================
;;;                            Conversation states
;;;===============================================================================

;;;--------------------------------------------------------- (MC) FIND-PERFORMATIVE

(defstate _mc-find-performative
  (:entry-state _process-conversation)
  (:process-state _main-conversation)
  (:label "Find performative")
  (:explanation 
   "Process what the user said trying to determine the type of ~
    performative among :request :assert :command. Put the result if any ~
	into the performative slot of the conversation object.")
  (:transitions 
   ;; time?
   (:patterns (("Quelle" "heure" (?* ?x)))
              :exec (print-time moss::conversation :fr) :success)
   (:patterns (("What" "time" (?* ?x)))
              :exec (print-time moss::conversation :fr) :success)
   ;; now really select performatives
   (:patterns (("what" "is" (?* ?x))
               ("what" "are" (?* ?x))
               ("what" (?* ?x))
               ("when" (?* ?x))
               ("where" (?* ?x))
               ("why" (?* ?x))
               ("who" (?* ?x))
               ("whom" (?* ?x))
               ("whose" (?* ?x))
               ("how" "much" (?* ?x))
               ("how" (?* ?x))
               ("is" (?* ?x))
               ("are" (?* ?x))
               ("were" (?* ?x))
               ("was" (?* ?x))
               ("have" (?* ?x))
               ("had" (?* ?x))
               ("do" (?* ?x))
               ("did" (?* ?x))
               ("can" (?* ?x))
               ("could" (?* ?x))
               ("may" (?* ?x))
               ("might" (?* ?x))
               ((?* ?x) "?")
               )
              :set-performative (list :request)
              :target _mc-find-task)
   (:patterns (((?* ?x) "note" (?* ?y))
               ((?* ?x) "remember" (?* ?y)))
              :set-performative (list :assert)
              :target _mc-find-task)
   (:otherwise 
    :set-performative (list :command)
    :target _mc-find-task))
  )

;;;--------------------------------------------------------------------- (MC) ELIZA

(moss::defstate _mc-ELIZA
  (:label "ELIZA")
  (:explanation
   "Whenever MOSS cannot interpret what the user is saying, ELIZA is ~
    called to do some meaningless conversation to keep the user happy. ~
    It then record the master's input and transfers it to find ~
	performative state.")
  (:eliza)
  (:transitions
   (:always :target _mc-find-performative))
  )

;;;------------------------------------------------------------------- (MC) FAILURE

(defstate _mc-failure
  (:label "Failure state of main conversation")
  (:explanation
   "Failure state is entered when we return from a sub-dialog with a ~
    :failure tag. We check for more tasks to perform (listed in the ~
	task-list slot of the conversation object). If there are more, we ~
	execute the first one. If there are no more, we return with a ~
	failure tag.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _mc-failure (conversation input)
  (declare (ignore input))
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         task)
    (if task-list
      (progn
        ;; record next task
        (setq task (pop task-list))
        ;; remove it from task-list
        (send conversation '=replace 'HAS-MOSS-TASK-LIST task-list)
        ;; add it to conversation
        (send conversation '=replace 'HAS-MOSS-TASK (list task))
        ;; transfer to sub-dialog
        `(:transition ,_mc-task-dialog))
      ;; when no more tasks we return :failure and the state variable still contains
      ;; :failure. The crawler will return one more level until hitting a non
      ;; failure return containing a specific state (e.g. the one for the main
      ;; conversation)
      (list :failure)
      )))

;;;----------------------------------------------------------------- (MC) FIND-TASK

(defstate _mc-find-task
  (:label "Find task")
  (:explanation 
   "Process the input to determine the task to be undertaken. Combines ~
    the words from the sentence to see if they are entry points for ~
	the index property of any task. Collect all tasks for which there ~
	is an index in the sentence.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _mc-find-task (conversation input)
  "Using input to find tasks If none failure, if one, OK, if more, ~
   must ask user to select one. Checks for an entry point for an index ~
   of a task.
Arguments:
   conversation: current conversation
   input: list containing the input as a list of words"
  (let* ((performative (read-fact conversation :PERFORMATIVE))
         task-list)
    (moss::vformat "_mc-find-task /performative: ~S, package: ~S, input:~%  ~S" 
	               performative *package* input)
    ;; try to find a task from the input text, combining input words
    (setq task-list (moss::find-objects 
                     '(TASK 
					    (HAS-INDEX-PATTERN 
					       (TASK-INDEX (HAS-INDEX :is :?))))
                     input :all-objects t)) 
	(moss::vformat "_mc-find-task /possible tasks:~%  ~S" task-list)
    ;; filter tasks that do not have the right performative
    (setq task-list
          (mapcan #'(lambda (xx) 
                      (if (intersection performative (HAS-PERFORMATIVE xx))
                        (list xx)))
                  task-list))
    (moss::vformat "_mc-find-task /task list after performative check: ~S" task-list)
    (cond 
     ;; if empty, ask ELIZA
     ((null task-list)
      `(:transition ,_mc-ELIZA))
     ;; if one, then OK
     ((null (cdr task-list))
      ;; save results, make the task the conversation task
      (setf (HAS-MOSS-TASK conversation) task-list)
      `(:transition ,_mc-task-dialog))
     (t 
      ;; otherwise must select one from the results
      (setf (HAS-MOSS-TASK-LIST conversation) task-list)
      `(:transition ,_mc-select-task)))
    ))

;;;--------------------------------------------------------------- (MC) SELECT-TASK

(defstate 
  _mc-select-task
  (:label "mc select task")
  (:explanation
    "we have located more than one task. We rank the tasks by computing ~
     the average weight of the terms in the task index slot. We then ~
     record the list of tasks into the conversation task-list slot ~
     and activate the highest ranking task.")
  (:answer-analysis) 
  )

(defownmethod =answer-analysis _mc-select-task (conversation input)
  "we have located more than one task. We rank the tasks by computing ~
   the MYCIN combination weight of the terms in the task index slot. ~
   We then record the list of tasks into the conversation task-list slot ~
   and activate the highest ranking task."
  ;; the list of tasks is in the HAS-MOSS-TASK-LIST slot. Tasks are defined in the
  ;; XXX-tasks.lisp file of the application package
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         patterns weights result pair-list selected-task level word-weight-list)
    (moss::vformat "_select-task /input: ~S~&  task-list: ~S" input task-list)
    ;; first compute a list of patterns (combinations of words) from the input
	;; 1-2-3-grams, i.e. max 3 words
    (setq patterns (mapcar #'car (moss::generate-access-patterns input)))
    (moss::vformat "_select-task /input: ~S~&  generated patterns:~&~S" 
                   input patterns)
    ;; then, for each task
    (dolist (task task-list)
      (setq level 0)
      ;; get the weight list (cached on the task p-list)
      (setq weights (moss::%get-INDEX-WEIGHTS task))
      (moss::vformat "_select-task /task: ~S~&  weights: ~S" task weights)
      ;; check the patterns according to the weight list
      (setq word-weight-list (moss::%get-relevant-weights weights patterns))  
      (moss::vformat "_select-task /word-weight-list:~&  ~S" word-weight-list)
      ;; combine the weights
      (dolist (item word-weight-list)
        (setq level (+ level (cadr item) (- (* level (cadr item))))))
      (moss::vformat "_select-task /level: ~S" level)
      ;; push the task and weight onto the result list
      (push (list task level) result)
      )
    (moss::vformat "_select-task /result:~&~S" result)
    ;; order the list
    (setq pair-list (sort result #'> :key #'cadr)) 
    (moss::vformat "_select-task /pair-list:~&  ~S" pair-list)
    ;; keep the first task whatever its score
    (setq selected-task (caar pair-list))
    ;; remove the task that have a weight less than task-threshold (default 0.4)
    (setq pair-list
          (remove nil 
                  (mapcar 
                   #'(lambda (xx) 
                       (if (>= (cadr xx) (omas::task-threshold omas::*omas*)) xx))
                   pair-list)))
    ;; if task-list is empty then return the first saved task
    ;; this may not be such a good policy if the score is too low
    (if (null pair-list)
      (progn
        ;; reset the task-list slot of the conversation object
        (setf (HAS-MOSS-TASK-LIST conversation) nil)
        ;; put the saved task into the task slot
        (setf (HAS-MOSS-TASK conversation) (list selected-task))
        ;; go to task-dialog
        `(:transition ,_mc-task-dialog)
        )
      (progn
        ;; remove the weights
        (setq task-list (mapcar #'car pair-list))
        ;; select the first task of the list
        (setq selected-task (pop task-list))
        (moss::vformat "_select-task /selected task: ~S" selected-task)
        ;; save the popped list in the task-list slot of the conversation object
        (setf (HAS-MOSS-TASK-LIST conversation) task-list)
        (setf (HAS-MOSS-TASK conversation) (list selected-task))
        ;; go to task-dialog
        `(:transition ,_mc-task-dialog)))
    ))

;;;--------------------------------------------------------------- (MC) TASK-DIALOG

(defstate _mc-task-dialog
  (:label "Task dialog")
  (:explanation 
   "We found a task to execute (in the GOAL slot of the conversation). ~
    We activate the dialog associated with this task.")
  ;; we launch the task dialog as a sub-conversation
  ;; the task contains the name of a sub-conversation header, e.g. _get-tel-nb
  ;; could be done within the defstate formalism
  (:answer-analysis)
  )

;;;********** must check for possible problems
(defownmethod
  =answer-analysis _mc-task-dialog (conversation input)
  "We prepare the set up to launch the task dialog.
Arguments:
   conversation: current conversation"
  (declare (ignore input))
  (let* ((task-id (car (send conversation '=get 'HAS-MOSS-TASK))))
    (moss::vformat "_mc-task-dialog /task-id: ~S dialog: ~S" 
                   task-id (HAS-DIALOG task-id))
    ;; we launch the task dialog as a sub-conversation
    ;; the task contains the name of a sub-conversation header, e.g. _get-tel-nb
    `(:sub-dialog ,(car (HAS-DIALOG task-id)) :failure ,_mc-failure)
    ))

;;;================================================================================
;;;
;;;                             ELIZA CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to do small talk.


(defParameter *eliza-rules*
  '((("hello")
     "Hello! I am TYLER."
     "Hello there?")
    (("Hi" (?* ?x))
     "Hi! Boss."
     "Hi!")
    (("bonjour")
     "Je suis désolé mon français n'est pas encore utilisable. Please use English.")
    (("how" "are" "you" (?* ?x))
     "Very well thank you."
     "Really does my health concern you? I am flattered.")
    (("who"  (?* ?x) "Barthès")
     "Jean-Paul Barthès is a professor of artificial intelligence at the ~
      Université de Technologie de Compiègne in France.")
    (("who"  (?* ?x) "Barthes")
     "Jean-Paul Barthès is a professor of artificial intelligence at the ~
      Université de Technologie de Compiègne in France.
      Note however that Barthès is written with an è.")
    (("forget" "it")
     "OK. No hard feelings.")
    (("thank" "you" (?* ?y))
     "you are welcome."
	 "don't mention it.")
    (("thanks")
     "you are welcome."
	 "don't mention it.")
    (((?* ?x) "computer" (?* ?y))
     "Do computer worry you?"
     "What do you think about machines?"
     "Why do you mention computers?"
     "What do you think machines have to do with your problem?")
    (((?* ?x) "stupid" (?* ?y))
     "Please, behave!"
     "I am pained by your remark.")
    (((?* ?x) "does" "not" "work" (?* ?y))
     "Of course ?x works!"
     "What are  you saying that ?x does not work?")
    (("help")
     "you want help about what?")
    (((?* ?x) "sorry" (?* ?y))
     "Please don't apologize"
     "Apologies are not necessary"
     "What feelings do you have when you apologize?")
    (((?* ?x) "I" "remember" (?* ?y))
     "Do you often think of ?y"
     "Does thinking of ?y bring anything else to mind?"
     "What else do you remember?"
     "Why do you recall ?y right now?"
     "What in the present situation reminds you of ?y ?"
     "What is the connection between me and ?y ?")
    (((?* ?x) "do" "you" "remember" (?* ?y))
     "Did you think I would forget ?y ?"
     "Why do you think I should recall ?y by now ?"
     "What about ?y ?"
     "You mentined ?y ?")
    (((?* ?x) "if" (?* ?y))
     "Do you really think its likely that ?y ?"
     "Do you wish that ?y ?"
     "What do you think about ?y ?"
     "Really -- if ?y ?")
    (("what" (?* ?x) "do" "for" "me")
     "So many things..."
     "Currently I do not know."
     "What could please you?")
    
    (((?* ?x) "I" "dreamt" (?* ?y))
     "Really -- ?y"
     "Have you ever fantasized ?y while you were awake ?"
     "Have you dreamt ?y before ?")
    (((?* ?x) "dream" "about" (?* ?y))
     "How do you feel about ?y in reality ?")
    (((?* ?x) "dream" (?* ?y))
     "What does this dream suggest to you ?"
     "Do you dream often ?"
     "What persons appear in your dreams ?"
     "Don't you believe that dream has to do with your problem ?")
    (((?* ?x) "my" "mother" (?* ?y))
     "Who else in your family ?y ?"
     "Tell me more about your family")
    (((?* ?x) "my" "father" (?* ?y))
     "Your father ?"
     "Does he influence you strongly ?"
     "What else comes to mind when you think of your father ?")
    (((?* ?x) "I" "want" (?* ?y))
     "What would it mean if you got ?y ?"
     "Why do you want ?y ?"
     "Suppose you got ?y soon ?")
    (((?* ?x) "I" "am" "glad" (?* ?y))
     "How have I helped you to be ?y ?"
     "What makes you happy just now ?"
     "Can you explain why you are suddenly ?y ?")
    (((?* ?x) "I" "am" "sad" (?* ?y))
     "I am sorry to hear you are depressed"
     "I am sure it's not pleasant to be sad")
    (((?* ?x) "are" "like" (?* ?y))
     "What resemblance do you see between ?x and ?y ?")
    (((?* ?x) "is" "like" (?* ?y))
     "In what way is it that ?x is like ?y ?"
     "What resemblance do you see ?"
     "Could there really be some connection ?"
     (How ?))
    (((?* ?x) "alike" (?* ?y))
     "In what way ?"
     "What similarities are there ?")
    (((?* ?x) "same" (?* ?y))
     "What other connections do you see ?")
    
    (((?* ?x) "I" "was" (?* ?y))
     "Were you really ?"
     "Perhaps I already knew you were ?y"
     "Why do you tell me you were ?y by now ?")
    (((?* ?x) "was" "I" (?* ?y))
     "What if you were ?y ?"
     "Do you think you were ?y ?"
     "What would it meant if you were ?y ?")
    (((?* ?x) "I" "am" (?* ?y))
     "In what way are you ?y ?"
     "Do you want to be ?y ?")
    (((?* ?x) "am" "I" (?* ?y))
     "Do you believe you are ?y ?"
     "Would you want to be ?y ?"
     "You wish I would tell you you are ?y"
     "What would it mean if you were ?y ?")
    (((?* ?x) "am" (?* ?y))
     "Why do you say \"AM?\""
     "I don't understand that")
    (((?* ?x) "are" "you" (?* ?y))
     "Why are you interested in whether I am ?y or not ?"
     "Would you prefer if I weren't ?y ?"
     "Perhaps I am ?y in your fantasies")
    (((?* ?x) "you" "are" (?* ?y))
     "What makes you think I am ?y ?")
    (((?* ?x) "because" (?* ?y))
     "Is that the real reason ?"
     "What other reasons might there be ?"
     "Does that reason seem to explain anything else ?")
    (((?* ?x) "were" "you" (?* ?y))
     "Perhaps I was ?y"
     "What do you think ?"
     "What if I had been ?y ?")
    (((?* ?x) "I" "can't" (?* ?y))
     "Maybe you could ?y now"
     "What if you could ?y ?")
    (((?* ?x) "I" "feel" (?* ?y))
     "Do you often feel ?y ?")
    (((?* ?x) "I" "felt" (?* ?y))
     "What other feelings do you have ?")
    (((?* ?x) "I" (?* ?y) "you" (?* ?z))
     "Perhaps in your fantasy we ?y each other")
    (((?* ?x) "why" "don't" "you" (?* ?y))
     "Should you ?y yourself ?"
     "Do you believe I don't ?y ?"
     "Perhaps I will ?y in good time")
    (((?* ?x) "yes" (?* ?y))
     "You seem quite positive."
     "You are sure ?"
     "I understand.")
    (((?* ?x) "no" (?* ?y))
     "Why not ?"
     "You are being a bit negative"
     "Are you saying \"NO\" just to be negative?")
    
    (((?* ?x) "someone" (?* ?y))
     "Can you be more specific ?")
    (((?* ?x) "everyone" (?* ?y))
     "Surely not everyone."
     "Con you think of anyone in particular ?"
     "Who for example ?"
     "You are thinking of a special person ?")
    (((?* ?x) "always" (?* ?y))
     "Can you think of a specific example ?"
     "When ?"
     "What incident are you thinking of ?"
     "Really -- always ?")
    (((?* ?x) "perhaps" (?* ?y))
     "You do not seem quite certain")
    (((?* ?x) "are" (?* ?y))
     "Did you think there might not be ?y ?"
     "Possibly they are ?y")
    (((?* ?x))
     "I do not understand exactly what you want. You may type: help."
     "I am not sure I understand what you want. You may type: help."
     "Could you rephrase your question please?"
     "Sorry I did not get that. Could you rephrase it? Or type: help."
     "Oops I do not understand. Could you say that differently?"
     "Sorry?"
     )))


#|

;;;========================== preparing mailing lists =============================

;;; Ask MORE INFO

(m-defstate _am-get-info ADDR
  (:label "More info")
  (:explanation "Assistant inferred that master wanted to obtain some sort ~
                 of info and has to find out what kind.")
  (:exec-var data)
  (:immediate-transitions
   (:starts-with (tel fax address email mail) :target _am-ask-address)
   (:starts-with (person paper) :target _am-ask-biblio)
   (:starts-with (mailing) :trim-data (data) :target _am-mailing-kind?)
   (:test (null data) :ask "What do you want exactly?")
   ;; otherwise should branch into the ask-around? state
   (:otherwise :target _am-ask-around?))
  (:transitions
   (:starts-with (tel fax address email mail) :target _am-ask-address)
   (:starts-with (person paper) :target _am-ask-biblio)
   (:starts-with (mailing) :trim-data (input) :target _am-mailing-kind?)))

;;; DETERMINE KIND OF MAILING

;;; missing =add-tp context 'HA?-MAILING-TYPE but in order to do that we should
;;; create and use a MAILING-CONTEXT object
;;; this could be done by creating a context type corresponding to the type of
;;; conversation taking place

(m-defstate _am-mailing-kind? ADDR
  (:label "Mailing kind?")
  (:explanation "We determine that master wants to do a mailing, we need to ~
                 know whether it is through email or postal mail.")
  (:exec-var data)
  (:immediate-transitions
   (:contains (email :email) :trim-data (data) 
              :trim-data (data) :set-context ('HAS-MAILING-TYPE '(email)) 
              :target _am-query-info)
   (:contains (postal mail :pmail) 
              :trim-data (data) :set-context ('HAS-MAILING-TYPE '(pmail)) 
              :target _am-query-info)
   (:otherwise
    :ask "what kind of mailing do you want: email or postal-mail?"))
  (:transitions
   (:contains (email :email) :set-context ('HAS-MAILING-TYPE '(email))
              :trim-data (input) :target _am-query-info)
   (:contains (postal mail :pmail) :set-context ('HAS-MAILING-TYPE '(email))
              :trim-data (input) :target _am-query-info)
   (:otherwise (:set-context ('HAS-MAILING-TYPE '(email)) :target _am-query-info))))

;;; QUERY INFO

(m-defstate _am-query-info ADDR
  (:label "Query info")
  (:explanation "We determined that we want to select mailing on a basis of a query ~
                 So we test if we have enough information to send to the address ~
                 agent for processing the query. Currently, we only check if there ~
                 is anything left in the data area. If so, this will be sued for ~
                 the query. ~
                 A query restricts the range of addresses to the result of a query ~
                 to the A-agent. Please formulate the query (example: \"Barthes\" ~
                 for getting all the persons named Barthès, or \"AFIA\" for getting ~
                 only the members of AFIA).")
  (:exec-var data)
  (:immediate-transitions
   (:contains (:help h ? how? how) :target _am-query-help)
   (:test data :target _am-send-query))
  (:ask "You should now formulate a question to restrict the range of addresses. 
Please formulate the query (example: all persons named \"Barthes\", or 
persons working-for \"AFIA\" for getting only the members of AFIA).")
  (:transitions
   (:contains (:help h ? how? how) :target _am-query-help)
   (:otherwise :save input :target _am-send-query)))

;;; QUERY HELP

(m-defstate _am-query-help ADDR
  (:label "Query help")
  (:explanation "help function to be able to formulate request. Indicates the ~
                 understood vocabulary.")
  (:ask "~&Possible vocabulary:
    all, person, company, firm, organization, name, named, location, living,
    living-at, working, working-for, at, employer, employed-by.
Possible expressions:
    person named \"Barthes\"
    person working-at \"UTC\"
    organization named \"IRIT\"
    person living \"Toulouse\"
    company at \"Compiegne\"
")
  (:transitions
   (:otherwise :save input :target _am-send-query)))

;;; SEND QUERY

;;; the comm process must deliver answer as a list for ust to be able to use the
;;; m-defstate macro

(m-defstate _am-send-query ADDR
  (:label "Send query")
  (:explanation "We collected info for producing a request, now we send it to the ~
                 SA-ADDRESS agent.")
  (:exec-var data)
  (:send-message)
  (:transitions
   (:test (and (typep (car input) 'message) (null (contents (car input))))
          :target _am-no-result)
   (:test (typep (car input) 'message) :save input 
          :target _am-process-results)))

;;; NO RESULT

(m-defstate _am-no-result ADDR
  (:label "No result")
  (:explanation "We asked SA-ADDRESS for a list of objects and got none. We should ~
                 rephrase the query.")
  (:ask "~& No answer to your query. Please rephrase it.
Possible vocabulary:
    all, person, company, firm, organization, name, named, location, living,
    living-at, working, working-for, at, employer, employed-by.
Possible expressions:
    person named \"Barthes\"
    person working-at \"UTC\"
    organization named \"IRIT\"
    person living \"Toulouse\"
    company at \"Compiegne\"
")
  (:transitions
   (:otherwise :save input :target _am-send-query)))

;;; PROCESS RESULTS

(m-defstate _am-process-results ADDR
  (:label "Process results")
  (:explanation "We got an answer from the SA-ADDRESS agent from which we must chosse.")
  (:exec-var data)
  (:ask-TSW :title "Select results"
            :choice-list (cddr (contents (car data)))
            :why "Select the objects that you want to keep on the mailing list.") 
  ;where do we put the result? in to-do, faking a question
  (:transitions
   (:otherwise :set-context ('HAS-RESULTS (list input)) 
               :target _am-get-mailing-index)))

;;; GET MAILING INDEX

(m-defstate _am-get-mailing-index ADDR
  (:label "Getting index")
  (:explanation "asking master to give a name for mailing list.")
  (:question "Please give me a name for this mailing list?")
  (:transitions
   (:otherwise :save input :target _am-save-mailing-list)))

|#


(mformat "~%;*** MOSS v~A - TYLER dialogs loaded ***" moss::*moss-version-number*)

;;;=============================================================================== 

:EOF