;;;-*- Mode: Lisp; Package: "CG-USER" -*-
;;;===============================================================================
;;;09/02/15
;;;       N O T E B O O K @ U T C <-> U T C - T E S T S (tcp-tests.lisp)
;;;
;;; Low level tests for checking remote TCP connections
;;;===============================================================================

#|
2009
 0113 creation
 0215 correcting some mistakes
|#

(in-package :cg-user)

(require :sock)

(defparameter *site* :NOTEBOOK-UTC)
(defparameter *site-ip* "172.17.157.115" "UTC portable (wire)")

(defparameter *site-port* 52008)

(defparameter *remote* :UTC)
(defparameter *remote-ip* "172.17.130.153" "UTC external ip")

;;;===============================================================================
;;;
;;;         Opening socket for letting other people connect to our site
;;;
;;;===============================================================================

(defparameter *receive-socket* nil)

(defun open-socket-for-connection ()
  (setq *receive-socket*
        (socket:make-socket :connect :passive :local-port *site-port* 
                            :backlog 50)))

;;;===============================================================================
;;;
;;;         Creating process to receive external messages
;;;
;;;===============================================================================

(defparameter *receiving-process* nil)
(defparameter *in-stream* nil)

(defun create-receiving-process ()
  (setq *receiving-process*
        (mp:process-run-function 
         (format nil "~S Receiving" *site*) #'receiving *site-port*)))

(defun receiving (port &aux message-string)
  "function called by the process"
  (loop
    ;; print trace into Lisp console
    (format t "~&=====~&~S/ receiving/ ready for incoming message..." *site*)
      
    ;; create new stream for each connection request 
    (setq *in-stream* (socket:accept-connection *receive-socket*)) 
    
    ;; get incoming message, returning nil if message is empty (:eof reached)
    (setq message-string (read *in-stream* nil nil))
    ;; close the stream after each received message
    (close *in-stream*)
    
    ;; print trace into Lisp console
    (format t "~&=====~&~S/ receiving/ incoming message: ~&     ~S"
      *site* message-string)
    ))

;;;===============================================================================
;;;
;;;                 Sending messages to remote site
;;;
;;;===============================================================================

(defvar *send-socket* nil)

(defparameter *count* 0)

(defun create-send-socket ()
  (setq *send-socket*
        (socket:make-socket :remote-host *remote-ip* :remote-port 52008)))

(defun close-send-socket ()
  (close *send-socket*)
  (setq *send-socket* nil)
  :done)

(defun send-new-message ()
  
  ;; if socket is opened close it
  (if *send-socket* (close *send-socket*))
  
  ;;; create socket-id
  (setq *send-socket* 
        (socket:make-socket :remote-host *remote-ip* :remote-port 52008))
  
  ;; if no error send message, writing message to stream
  (format *send-socket* "~S" 
    (format nil "Test message from ~S #~S." *site* (incf *count*)))
  
  ;; make sure the buffer is emptied
  (force-output *send-socket*)
  
  ;; close the socket before leaving
  (close *send-socket*)
  )

;;;===============================================================================
;;;
;;;         Killing process and closing socket for reset
;;;
;;;===============================================================================

(defun reset-everything ()
  (when *in-stream* (close *in-stream*)
    (setq *in-stream* nil))
  (when *receive-socket* (close *receive-socket*)
    (setq *receive-socket* nil))
  (when *receiving-process* (mp:process-kill *receiving-process*)
    (setq *receiving-process* nil))
  (when *send-socket* (close *send-socket*)
    (setq *send-socket* nil))
  )

;;;===============================================================================
;;;
;;;         Tests
;;;
;;;===============================================================================

#|
;;; create a socket for receiving messages
open-socket-for-connection

*receive-socket*

;;; create process and allow clients to connect on the socket
(create-receiving-process)

*receiving-process*

;;; this prints a message in the console:
;;;
;;; =====
;;; :NOTEBOOK-UTC/ receiving/ ready for incoming message...
;;; 

;;; can check processes that a new process has been created and is waiting
;;; process name is ":NOTEBOOK-UTC Receiving"

;;;========== SENDING

;;; try opening a socket on remote site prior to sending a message
;;; Remote site should have a socket waiting for a connection on the socket
(create-send-socket)

;;; if it does not work, it means that the remote site is not available or is
;;; protected by a firewall, or there is an error in the IP or in the port number
;;; Try to do a ping for checking the availabiity of the remote machine

;;; If it works, the socket is recorded in the following variable:
*send-socket* 

;;; CLOSE IT before trying to send a message
(close-send-socket)

;;; Now try to send a message: 
;;;     "Test message from <local-site> #nn."

(send-new-message)

;;; The receiving function on the other site should print into the Lisp console:
;;;
;;; "=====
;;;  NOTEBOOK-UTC/ receiving/ incoming message: 
;;;       Test message from <remote-site> #nn.

;;; If everyting is OK, on can send as many messages as one wants

;;; If something goes wrong the following function resets everything

(reset-everything)

|#
:EOF